package robots.comm;

import auto.Automaton;
import physics.CalcObject;
import physics.PhysicalObject;
import robots.field.CoordinateObject;

/**
 * This class is used for testing the communicating possibilities of the Infrared Ports.
 * If it has received the message it waits for some time and after that starts retransmitting it for the specified time.
 *
 * @author Boris Yartsev
 */
public class IRDACommunicatorTester implements Automaton, CalcObject, CoordinateObject {
    public static final int WAITING_STATE = 1;
    public static final int WAITING_AFTER_RECEIVER_STATE = 2;
    public static final int TRANSMITTING_STATE = 3;

    public static final int SIGNAL_RECEIVED_EVENT = 0;
    public static final int SEND_TIMER_EVENT = 1;
    public static final int WAIT_TIMER_EVENT = 2;
    private int state = WAITING_STATE;

    public static final double WAIT_PERIOD = 1;
    public static final double SEND_PERIOD = 0.5;

    private InfraredReceiverEmitter recem;
    private byte recSignal = 0;

    public IRDACommunicatorTester() {
        recem = new InfraredReceiverEmitter();
        recem.getReceiver().addAutomatonUpdatorOnReceiving(this, SIGNAL_RECEIVED_EVENT);
        recem.getReceiver().enable();

        waitPeriod = WAIT_PERIOD;
        sendPeriod = SEND_PERIOD;
    }

    public double getX() {
        return recem.getX();
    }

    public void setX(double x) {
        recem.setX(x);
    }

    public double getY() {
        return recem.getY();
    }

    public void setY(double y) {
        recem.setY(y);
    }

    public double getAngle() {
        return recem.getAngle();
    }

    public void setAngle(double angle) {
        recem.setAngle(angle);
    }

    public InfraredReceiverEmitter getReceiverEmitter() {
        return recem;
    }

    public synchronized void updateAutoState(int event, int data) {
        switch(getAutoState()) {
            case WAITING_STATE: {
                switch(event) {
                    case SIGNAL_RECEIVED_EVENT:
                        z3_storeSignal();
                        z4_restartWaitTimer();
                        z6_lockReceiver();
                        setAutoState(WAITING_AFTER_RECEIVER_STATE);
                        break;
                }

                break;
            }

            case WAITING_AFTER_RECEIVER_STATE: {
                switch(event) {
                    case WAIT_TIMER_EVENT:
                        z1_startSending(recSignal);
                        z5_restartSendTimer();
                        setAutoState(TRANSMITTING_STATE);
                        break;
                }

                break;
            }

            case TRANSMITTING_STATE: {
                switch(event) {
                    case SEND_TIMER_EVENT:
                        z2_stopSending();
                        z7_unlockReceiver();
                        setAutoState(WAITING_STATE);
                        break;
                }
                break;
            }
        }
    }

    private void z1_startSending(byte value) {
        recem.getEmitter().startSending(value);
    }

    private void z2_stopSending() {
        recem.getEmitter().stopSending();
    }

    private void z3_storeSignal() {
        recSignal = recem.getReceiver().getMessage();
    }

    private void z6_lockReceiver() {
        recem.getReceiver().disable();
    }

    private void z7_unlockReceiver() {
        recem.getReceiver().enable();
    }

    public int getAutoState() {
        return state;
    }

    public void setAutoState(int state) {
        this.state = state;
    }

    //------------------------------------------------------------------------------------------------------------------
    //------------------------------------------------------------------------------------------------------------------
    //these two methods are used for updating the calc state of the automaton

    private int calcState = DEFAULT_CALC_STATE;

    public static final int DEFAULT_CALC_STATE = 1;

    private double timer = 0;

    private double waitTimerStart = -1;
    private double sendTimerStart = -1;

    private double waitPeriod;
    private double sendPeriod;

    public int getCalcState() {
        return calcState;
    }

    public void setCalcState(int calcState) {
        this.calcState = calcState;
    }

    public void updateCalcState(int calcEvent, int data, double timeDelta, Object objectData) {
        switch(getCalcState()) {
            case DEFAULT_CALC_STATE: {
                switch(calcEvent) {
                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        boolean x11 = x11_waitTimerReached();
                        boolean x12 = x12_sendTimerReached();

                        if (x11) {
                            z4_restartWaitTimer();
                            updateAutoState(WAIT_TIMER_EVENT, 0);
                        }

                        if (x12) {
                            z5_restartSendTimer();
                            updateAutoState(SEND_TIMER_EVENT, 0);
                        }

                        z11_updateTimers(timeDelta);
                        break;
                }
                break;
            }
        }
    }

    private boolean x11_waitTimerReached() {
        return timer > waitPeriod + waitTimerStart;
    }

    private boolean x12_sendTimerReached() {
        return timer > sendPeriod + sendTimerStart;
    }

    private void z11_updateTimers(double delta) {
        timer += delta;
    }

    private void z4_restartWaitTimer() {
        waitTimerStart = timer;
    }

    private void z5_restartSendTimer() {
        sendTimerStart = timer;
    }

    public String toString() {
        return "CommTester:" + " State = " + getAutoState() + " CalcState = " + getCalcState() + ": " + recem.toString();
    }
}