package robots.comm;

import physics.PhysicalObject;
import physics.CalcObject;

import auto.Automaton;

import java.awt.*;

import robots.view.DrawableObject;
import robots.field.CoordinateObject;

/**
 * @author Boris Yartsev
 */
public class InfraredEmitter implements Automaton, CalcObject, DrawableObject, CoordinateObject {
    private double infraredRadius = 150;
    private double infraredAlpha = Math.PI / 4;
    private Color emitterColor = Color.RED;

    //The period between sending two signals
    private double emittingPeriod = 0.1;

    private int state = IDLE_STATE;
    public static final int IDLE_STATE = 1;
    public static final int SENDING_NOT_READY_STATE = 2;
    public static final int SENDING_READY_STATE = 3;

    public static final int E1_START_SENDING_EVENT = 1;
    public static final int E2_END_SENDING_EVENT = 2;
    public static final int E3_SIGNAL_TAKEN_EVENT = 3;
    public static final int E4_SENDING_READY_EVENT = 4;

    private byte sendingValue = 0;

    /**
     * These are the coordinates of the reciever on the field, they are updated by the robot,
     * the reciever belongs to
     */
    private double x = 0;
    private double y = 0;
    private double angle = 0;

    private String emitterImageLocation = "images\\robots\\irdaem.GIF";
    private Image emitterImage = Toolkit.getDefaultToolkit().createImage(emitterImageLocation);
    private boolean shown = true;


    public InfraredEmitter() {
        this(150, Math.PI / 4, 0.1);
    }

    /**
     * The emitter is constructed with three parameters
     * @param radius
     * @param alpha
     * @param emittingPeriod
     */
    public InfraredEmitter(double radius, double alpha, double emittingPeriod) {
        this(Color.RED, radius, alpha, emittingPeriod);
    }

    public InfraredEmitter(Color emitterColor, double radius, double alpha, double emittingPeriod) {
        this.emitterColor = emitterColor;
        this.infraredRadius = radius;
        this.infraredAlpha = alpha;
        this.emittingPeriod = emittingPeriod;
    }

    public synchronized void updateAutoState(int event, int data) {
        switch(getAutoState()) {
            case IDLE_STATE:
                switch(event) {
                    case E1_START_SENDING_EVENT:
                        z0_setSendingValue((byte) data);
                        setAutoState(SENDING_NOT_READY_STATE);
                        break;
                }

                break;

            case SENDING_NOT_READY_STATE:
                switch(event) {
                    case E4_SENDING_READY_EVENT:
                        setAutoState(SENDING_READY_STATE);
                        break;

                    case E2_END_SENDING_EVENT:
                        setAutoState(IDLE_STATE);
                        break;
                }

                break;

            case SENDING_READY_STATE:
                switch(event) {
                    case E3_SIGNAL_TAKEN_EVENT:
                        setAutoState(SENDING_NOT_READY_STATE);
                        break;

                    case E2_END_SENDING_EVENT:
                        setAutoState(IDLE_STATE);
                        break;
                }
                break;
        }
    }

    /**
     * Start sending the value to the other objects on the field, equipped with the infrared sensors
     * @param value
     */
    public void startSending(byte value) {
        updateAutoState(E1_START_SENDING_EVENT, value);
        updateCalcState(E3_START_EMITTING_TIMER_CALC_EVENT, 0, 0, null);
    }

    public void stopSending() {
        updateAutoState(E2_END_SENDING_EVENT, 0);
        updateCalcState(E2_STOP_EMITTING_TIMER_CALC_EVENT, 0, 0, null);
    }

    private void z0_setSendingValue(byte value) {
        this.sendingValue = value;
    }


    public int getAutoState() {
        return state;
    }

    public void setAutoState(int state) {
        this.state = state;
    }

    public double getInfraredRadius() {
        return infraredRadius;
    }

    public double getInfraredAlpha() {
        return infraredAlpha;
    }

    public boolean isSignalReady() {
        return getAutoState() == SENDING_READY_STATE;
    }

    public boolean isSending() {
        return (getAutoState() == SENDING_READY_STATE) || (getAutoState() == SENDING_NOT_READY_STATE);
    }

    public byte getSendingValue() {
        return sendingValue;
    }

    /**
     * This method should be called to inform the emitter, that the signal has been transmitted
     */
    public void signalSent() {
        updateAutoState(E3_SIGNAL_TAKEN_EVENT, 0);
        updateCalcState(E1_FLUSH_EMITTING_TIMER_CALC_EVENT, 0, 0, null);
    }


    public double getX() {
        return x;
    }

    public void setX(double x) {
        this.x = x;
    }

    public double getY() {
        return y;
    }

    public void setY(double y) {
        this.y = y;
    }

    public double getAngle() {
        return angle;
    }

    public void setAngle(double angle) {
        this.angle = angle;
    }

    //------------------------------------------------------------------------------------------------------------------
    public static final int EMITTING_TIMER_NOT_ACTIVE_CALC_STATE = 1;
    public static final int EMITTING_TIMER_ACTIVE_CALC_STATE = 2;


    public static final int E1_FLUSH_EMITTING_TIMER_CALC_EVENT = 1;
    public static final int E2_STOP_EMITTING_TIMER_CALC_EVENT = 2;
    public static final int E3_START_EMITTING_TIMER_CALC_EVENT = 3;

    private int calcState = EMITTING_TIMER_NOT_ACTIVE_CALC_STATE;

    //This section covers the stuff with the calculus
    private double emittingTimerValue = 0;

    //these two methods are used for updating the calc state of the automaton
    public int getCalcState() {
        return calcState;
    }

    public void setCalcState(int calcState) {
        this.calcState = calcState;
    }

    public synchronized void updateCalcState(int calcEvent, int data, double timeDelta, Object objectData) {
        switch(getCalcState()) {
            case EMITTING_TIMER_NOT_ACTIVE_CALC_STATE: {
                switch(calcEvent) {
                    case E3_START_EMITTING_TIMER_CALC_EVENT:
                        setCalcState(EMITTING_TIMER_ACTIVE_CALC_STATE);
                        break;
                }

                break;
            }

            case EMITTING_TIMER_ACTIVE_CALC_STATE: {
                switch(calcEvent) {
                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        boolean x10 = x10_timerExceededEmittingPeriod();

                        if (x10) {
                            z10_updateTimerValue(timeDelta);
                            z12_updateControlAutomaton();
                        } else

                        if (!x10) {
                            z10_updateTimerValue(timeDelta);
                        }

                        break;

                    case E1_FLUSH_EMITTING_TIMER_CALC_EVENT:
                        z11_restartTimer();
                        break;

                    case E2_STOP_EMITTING_TIMER_CALC_EVENT:
                        z11_restartTimer();
                        setCalcState(EMITTING_TIMER_NOT_ACTIVE_CALC_STATE);
                        break;
                }

                break;
            }
        }
    }

    private boolean x10_timerExceededEmittingPeriod() {
        return emittingTimerValue > emittingPeriod;
    }

    private void z10_updateTimerValue(double delta) {
        emittingTimerValue += delta;
    }

    private void z11_restartTimer() {
        emittingTimerValue = 0;
    }

    private void z12_updateControlAutomaton() {
        updateAutoState(E4_SENDING_READY_EVENT, 0);
    }

    //------------------------------------------------------------------------------------------------------------------
    //------------------------------------------------------------------------------------------------------------------
    /*
    public void setObjectImage(Image img) {
        zoomer = new ImageZoomer(img);
    }
    */

    public Image getObjectImage() {
        return emitterImage;
    }

    public void setObjectImage(Image img) {
        this.emitterImage = img;
    }

    /*
    public Image getObjectImage() {
        return zoomer.getBufferImageOfFactor(zoomRatio);
    }
    */

    public boolean isShown() {
        return shown;
    }

    public void setShown(boolean shown) {
        this.shown = shown;
    }

    public Color getEmitterColor() {
        return emitterColor;
    }

    public String toString() {
        return "Emitter: " + " State = " + getAutoState() + " CalcState = " + getCalcState() + "; ";
    }
}