package robots.comm;

import physics.CalcObject;
import physics.PhysicalObject;
import auto.Automaton;
import robots.field.CoordinateObject;

/**
 * This class is used for emulation of the infrared transmitter used in Lego MindStorm
 * @author Boris Yartsev
 */
public class RobotIRDAModule implements Automaton, CalcObject, CoordinateObject {
    public static final int IDLE_STATE = -1;
    public static final int WAITING_STATE = 1;
    public static final int SENDING_STATE = 2;

    public static final int E1_SEND_MESSAGE_EVENT = -1;
    public static final int E2_SIGNAL_RECEIVED_EVENT = 0;
    public static final int E3_SEND_TIMER_EVENT = 1;
    public static final int E4_WAIT_TIMER_EVENT = 2;
    public static final int E5_RESTART_EVENT = 3;

    private int state = IDLE_STATE;

    public static final double WAIT_PERIOD = 0.8;
    public static final double SEND_PERIOD = 1;

    private InfraredReceiverEmitter recem;
    private int recSignal = -1;
    private byte sendValue = 0;

    public RobotIRDAModule() {
        recem = new InfraredReceiverEmitter();
        recem.getReceiver().addAutomatonUpdatorOnReceiving(this, E2_SIGNAL_RECEIVED_EVENT);
        recem.getReceiver().enable();

        waitPeriod = WAIT_PERIOD;
        sendPeriod = SEND_PERIOD;
    }

    public double getX() {
        return recem.getX();
    }

    public void setX(double x) {
        recem.setX(x);
    }

    public double getY() {
        return recem.getY();
    }

    public void setY(double y) {
        recem.setY(y);
    }

    public double getAngle() {
        return recem.getAngle();
    }

    public void setAngle(double angle) {
        recem.setAngle(angle);
    }

    public InfraredReceiverEmitter getReceiverEmitter() {
        return recem;
    }

    public void sendMessage(byte message) {
        updateAutoState(E1_SEND_MESSAGE_EVENT, message);
    }

    public boolean isAbleToSend() {
        return getAutoState() == IDLE_STATE;
    }

    public boolean isMessageFromRemoteReceived() {
        if (recSignal == -1) {
            return false;
        }

        return ((recSignal & 128) == 0);
    }

    public boolean isMessageFromRobotReceived() {
        if (recSignal == -1) {
            return false;
        }

        return ((recSignal & 128) > 0);
    }

    public int readMessage() {
        int message = recSignal;
        recSignal = -1;

        if (message == -1) {
            return -1;
        } else {
            return (message & 127);
        }
    }

    public void flushMessages() {
        readMessage();
    }

    public synchronized void updateAutoState(int event, int data) {
        switch(getAutoState()) {
            case IDLE_STATE:
                switch(event) {
                    case E1_SEND_MESSAGE_EVENT:
                        z4_restartWaitTimer();
                        z7_storeMessage((byte) data);
                        setAutoState(WAITING_STATE);
                        break;

                    case E2_SIGNAL_RECEIVED_EVENT:
                        z3_storeSignal();
                        break;

                    case E5_RESTART_EVENT:
                        z2_stopSending();
                        break;
                }
                break;


            case WAITING_STATE: {
                switch(event) {
                    case E2_SIGNAL_RECEIVED_EVENT:
                        z3_storeSignal();
                        break;

                    case E4_WAIT_TIMER_EVENT:
                        z5_restartSendTimer();
                        z1_startSending();
                        setAutoState(SENDING_STATE);
                        break;

                    case E5_RESTART_EVENT:
                        z2_stopSending();
                        setAutoState(IDLE_STATE);
                        break;
                }

                break;
            }

            case SENDING_STATE: {
                switch(event) {
                    case E2_SIGNAL_RECEIVED_EVENT:
                        z3_storeSignal();
                        break;

                    case E3_SEND_TIMER_EVENT:
                        z2_stopSending();
                        setAutoState(IDLE_STATE);
                        break;

                    case E5_RESTART_EVENT:
                        z2_stopSending();
                        setAutoState(IDLE_STATE);
                        break;
                }
                break;
            }
        }
    }

    private void z1_startSending() {
        recem.getEmitter().startSending(sendValue);
    }

    private void z2_stopSending() {
        recem.getEmitter().stopSending();
    }

    private void z3_storeSignal() {
        recSignal = recem.getReceiver().getMessage();
    }

    private void z7_storeMessage(byte message) {
        sendValue = message;
    }

    public int getAutoState() {
        return state;
    }

    public void setAutoState(int state) {
        this.state = state;
    }

    //------------------------------------------------------------------------------------------------------------------
    //------------------------------------------------------------------------------------------------------------------
    //these two methods are used for updating the calc state of the automaton

    private int calcState = DEFAULT_CALC_STATE;

    public static final int DEFAULT_CALC_STATE = 1;

    private double timer = 0;

    private double waitTimerStart = -1;
    private double sendTimerStart = -1;

    private double waitPeriod;
    private double sendPeriod;

    public int getCalcState() {
        return calcState;
    }

    public void setCalcState(int calcState) {
        this.calcState = calcState;
    }

    public void updateCalcState(int calcEvent, int data, double timeDelta, Object objectData) {
        switch(getCalcState()) {
            case DEFAULT_CALC_STATE: {
                switch(calcEvent) {
                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        boolean x11 = x11_waitTimerReached();
                        boolean x12 = x12_sendTimerReached();

                        if (x11) {
                            z4_restartWaitTimer();
                            z11_updateTimers(timeDelta);
                            updateAutoState(E4_WAIT_TIMER_EVENT, 0);
                        } else

                        if (x12) {
                            z5_restartSendTimer();
                            z11_updateTimers(timeDelta);
                            updateAutoState(E3_SEND_TIMER_EVENT, 0);
                        } else

                        if (!x11 && !x12) {
                            z11_updateTimers(timeDelta);
                        }

                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        z4_restartWaitTimer();
                        z5_restartSendTimer();
                        updateAutoState(E5_RESTART_EVENT, 0);
                        break;
                }
                break;
            }
        }
    }

    private boolean x11_waitTimerReached() {
        return timer > waitPeriod + waitTimerStart;
    }

    private boolean x12_sendTimerReached() {
        return timer > sendPeriod + sendTimerStart;
    }

    private void z11_updateTimers(double delta) {
        timer += delta;
    }

    private void z4_restartWaitTimer() {
        waitTimerStart = timer;
    }

    private void z5_restartSendTimer() {
        sendTimerStart = timer;
    }

    public String toString() {
        return "RobotIRDA:" + " State = " + getAutoState() + " CalcState = " + getCalcState() + ": " + recem.toString();
    }
}
