package robots.dispencer;

import physics.PhysicalObject;
import physics.CalcObject;
import auto.ControlAutomaton;
import robots.comm.RobotIRDAModule;
import robots.sensors.TouchSensor;
import robots.view.MandMFactory;
import robots.FieldRobot;
import robots.ControlSystem;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.*;
import java.util.List;

/**
 * @author Boris Yartsev
 */
public class Dispencer extends PhysicalObject implements FieldRobot {
    public static final int DISPENCING_CALC_STATE = 1;
    public static final int STOP_CALC_STATE = 2;

    public static final int E1_START_DISPENCING_CALC_EVENT = 1;
    public static final int E2_STOP_DISPENCING_CALC_EVENT = 2;

    private TouchSensor touchSensor;

    private RobotIRDAModule irdaModule;
    private double irdaModuleRadius = 22;
    private double irdaModuleAngle = 7 * Math.PI / 4;
    private double irdaModuleDirection = Math.PI;

    private DispencerControlSystem system = null;

    private List sensors;

    public static final int MAX_POWER = 7;
    private int power = 0;

    /**
     * The dispencer hook position varies from the 0 to the 2 * Pi
     */
    private double dispencerHookPosition = 0;

    private Conveyor conveyor = null;
    private int cX[] = {
        40, 30, 20, 18, 18, 18
    };

    private int cY[] = {
        12, 12, 12, 18, 25, 32
    };

    private String dispencerImageLocation = "images\\robots\\dispencer.gif";
    private Image dispencerImage = Toolkit.getDefaultToolkit().createImage(dispencerImageLocation);
    private Image dispencerBufferImage = null;

    private Image greenImage = MandMFactory.createMandM(MandMFactory.MANDM_GREEN);
    private Image redImage = MandMFactory.createMandM(MandMFactory.MANDM_RED);
    private Image yellowImage = MandMFactory.createMandM(MandMFactory.MANDM_YELLOW);
    private Image blueImage = MandMFactory.createMandM(MandMFactory.MANDM_BLUE);


    private boolean shown = true;

    //the real time of the system
    private double time = 0;

    private String description = "Dispencer robot";
    private String lcdMessage = "";

    public Dispencer() {
        super(0, 0, 0);

        sensors = new ArrayList();

        touchSensor = new TouchSensor("Dispencer touch sensor");
        sensors.add(touchSensor);

        irdaModule = new RobotIRDAModule();
        irdaModule.getReceiverEmitter().setShown(false);

        system = new DispencerControlSystem();

        conveyor = new Conveyor(cX, cY);

        setCalcState(STOP_CALC_STATE);
    }

    public RobotIRDAModule getIrdaModule() {
        return irdaModule;
    }

    public void startDispencing(int power) {
        updateCalcState(E1_START_DISPENCING_CALC_EVENT, power, 0, null) ;
    }

    public void stopDispencing() {
        updateCalcState(E2_STOP_DISPENCING_CALC_EVENT, 0, 0, null);
    }

    public void restart() {
        updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
    }

    public int getPower() {
        return power;
    }

    public List getSensors() {
        return sensors;
    }

    public ControlSystem getControlSystem() {
        return system;
    }

    public String getDescription() {
        return description;
    }

    public String getLcdMessage() {
        return lcdMessage;
    }

    //This weird procedure is added to make the setting of coordinates for different objects, that surround transport robot
    public void setX(double x) {
        this.x = x;
        setAngle(angle);
    }

    public void setY(double y) {
        this.y = y;
        setAngle(angle);
    }

    public void setAngle(double angle) {
        this.angle = angle;
        updateIrdaCoords();
    }

    private void updateIrdaCoords() {
        irdaModule.setX(x + irdaModuleRadius * Math.cos(angle + irdaModuleAngle));
        irdaModule.setY(y + irdaModuleRadius * Math.sin(angle + irdaModuleAngle));
        irdaModule.setAngle(angle + irdaModuleDirection);
    }

    public Image getObjectImage() {
        if (dispencerImage.getWidth(null) <= 0 || dispencerImage.getHeight(null) <= 0) {
            return null;
        }

        if (dispencerBufferImage == null) {
            dispencerBufferImage = new BufferedImage(dispencerImage.getWidth(null),
                    dispencerImage.getHeight(null), BufferedImage.TYPE_INT_ARGB);
        }

        Graphics bg = dispencerBufferImage.getGraphics();
        bg.drawImage(dispencerImage, 0, 0, null);

        for (int i = 0; i < conveyor.getMandMNumber(); i++) {
            int x = conveyor.getXPoint(i);
            int y = conveyor.getYPoint(i);
            int type = conveyor.getType(i);

            Image img = null;

            switch(type) {
                case MandMFactory.MANDM_GREEN:
                    img = greenImage;
                    break;

                case MandMFactory.MANDM_BLUE:
                    img = blueImage;
                    break;

                case MandMFactory.MANDM_RED:
                    img = redImage;
                    break;

                case MandMFactory.MANDM_YELLOW:
                    img = yellowImage;
                    break;
            }

            if (img != null) {
                bg.drawImage(img, x - img.getWidth(null) / 2,
                        y - img.getHeight(null) / 2, null);
            }
        }

        return dispencerBufferImage;
    }

    public void setObjectImage(Image img) {
    }

    public boolean isShown() {
        return shown;
    }

    public void setShown(boolean shown) {
        this.shown = shown;
    }

    public synchronized void updateCalcState(int event, int data, double timeDelta, Object objData) {
        switch(getCalcState()) {
            case DISPENCING_CALC_STATE: {
                switch(event) {
                    case E2_STOP_DISPENCING_CALC_EVENT:
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        irdaModule.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        system.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);

                        z2_updateTime(timeDelta);
                        z3_updateCoordinates(timeDelta);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        irdaModule.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
                        system.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
                        z1_restart();
                        setCalcState(STOP_CALC_STATE);
                        break;
                }
                break;
            }

            case STOP_CALC_STATE: {
                switch(event) {
                    case E1_START_DISPENCING_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(DISPENCING_CALC_STATE);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        irdaModule.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        system.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);

                        z2_updateTime(timeDelta);
                        z3_updateCoordinates(timeDelta);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        irdaModule.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
                        system.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
                        z1_restart();
                        break;
                }
                break;
            }
        }
    }

    private void z0_setPower(int power) {
        if (power > MAX_POWER) {
            power = MAX_POWER;
        }

        if (power < 0) {
            power = 0;
        }

        this.power = power;
    }

    private void z1_restart() {
        dispencerHookPosition = 0;
        wTorque = 0;
        lcdMessage = "";
        time = 0;
    }

    private void z2_updateTime(double timeDelta) {
        time += timeDelta;
    }

    /**
     * This method performs the recalculation of the parameters of the robot and the wheels.
     * The time is measured in seconds
     * @param timeDelta - the period of time, that passed since the last moment of emulation
     */
    private void z3_updateCoordinates(double timeDelta) {
        switch(getCalcState()) {
            case STOP_CALC_STATE: {
                wTorque = 0;
                conveyor.setDelta(dispencerHookPosition / (2 * Math.PI));
                break;
            }

            case DISPENCING_CALC_STATE: {
                wTorque = 0.1 * getPower() * Math.PI;
                dispencerHookPosition += wTorque * timeDelta;

                boolean dispence = dispencerHookPosition > Math.PI;
                dispencerHookPosition %= (2 * Math.PI);
                if (dispencerHookPosition < Math.PI / 4 && dispence) {
                    conveyor.dispence();
                }


                conveyor.setDelta(dispencerHookPosition / (2 * Math.PI));

                if (dispencerHookPosition > 3 * Math.PI / 2) {
                    touchSensor.touch();
                } else {
                    touchSensor.untouch();
                }

                break;
            }
        }
    }

    public Conveyor getConveyor() {
        return conveyor;
    }

    public class Conveyor {
        private int x[];
        private int y[];
        private int type[];

        private int mandmNumber = 0;

        private double delta;

        private Random rand = new Random(System.currentTimeMillis());

        private boolean itemReady = false;
        private int itemReadyType;

        public Conveyor(int x[], int y[]) {
            this.x = x;
            this.y = y;
            mandmNumber = x.length - 1;
            type = new int[mandmNumber];

            for (int i = 0; i < mandmNumber; i++) {
                type[i] = getRandType();
            }
        }

        public boolean isItemReady() {
            return itemReady;
        }

        public int getItemReady() {
            if (itemReady) {
                itemReady = false;
                return itemReadyType;
            } else {
                return -1;
            }
        }

        public void setDelta(double delta) {
            this.delta = delta;
        }

        private int getRandType() {
            int r = rand.nextInt(10000) % 4;
            int t = -1;
            switch(r) {
                case 0:
                    t = MandMFactory.MANDM_BLUE;
                    break;

                case 1:
                    t = MandMFactory.MANDM_GREEN;
                    break;

                case 2:
                    t = MandMFactory.MANDM_RED;
                    break;

                case 3:
                    t = MandMFactory.MANDM_YELLOW;
                    break;
            }

            return t;
        }

        public void dispence() {
            delta = 0;

            itemReadyType = type[mandmNumber - 1];
            itemReady = true;

            int oldType = type[0];
            for (int i = 1; i < mandmNumber; i++) {
                int t = type[i];
                type[i] = oldType;
                oldType = t;
            }

            type[0] = getRandType();
        }

        public int getMandMNumber() {
            return mandmNumber;
        }


        /**
         * The point number should be one less, than the number of the points loaded to the conveyor
         * @param pointNumber
         * @return
         */
        public int getXPoint(int pointNumber) {
            if (pointNumber > x.length - 2 || pointNumber < 0) {
                return -1;
            }

            double val = x[pointNumber] + delta * (x[pointNumber + 1] - x[pointNumber]);
            return (int) val;
        }

        public int getYPoint(int pointNumber) {
            if (pointNumber > x.length - 2 || pointNumber < 0) {
                return -1;
            }

            double val = y[pointNumber] + delta * (y[pointNumber + 1] - y[pointNumber]);
            return (int) val;
        }

        public int getType(int pointNumber) {
            if (pointNumber > x.length - 2 || pointNumber < 0) {
                return -1;
            }

            return type[pointNumber];
        }
    }

    public class DispencerControlSystem implements CalcObject, ControlSystem {
        private int state = CONTROL_CALC_STATE;

        public static final int CONTROL_CALC_STATE = 0;

        private double timer0 = 0;
        private boolean timer0active = false;

        public static final int A0_ID = 0;

        private ControlAutomaton a0;
        private List automata;

        public DispencerControlSystem() {
            automata = new ArrayList();

            a0 = new A0Auto(A0_ID);
            automata.add(a0);
        }

        public int getCalcState() {
            return state;
        }

        public void setCalcState(int state) {
            this.state = state;
        }

        public void restart() {
            system.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
        }

        public List getControlAutomata() {
            return automata;
        }

        /**
         * Here data is the time in the milliseconds
         * @param event
         * @param data
         * @param delta
         * @param obj
         */
        public void updateCalcState(int event, int data, double delta, Object obj) {
            switch (getCalcState()) {
                case CONTROL_CALC_STATE: {
                    switch(event) {
                        case PhysicalObject.E101_RESTART_CALC_EVENT:
                            z3_restartControlSystem();
                            break;

                        case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                            z0_updateTimers(delta);
                            z2_updateControlSystemState();
                            break;
                    }
                    break;
                }
            }
        }

        private long prevTime = -1;
        private double currentTime = 0;

        private void z0_updateTimers(double timeDouble) {
            currentTime += timeDouble;

            long time = (long) (1000 * currentTime);

            if (prevTime == -1) {
                prevTime = time;
            }

            if (timer0active) {
                timer0 += time - prevTime;
            }

            prevTime = time;
        }

        private void z2_updateControlSystemState() {
            a0.updateAutoState(0, 0);
        }

        private void z3_restartControlSystem() {
            a0.init();
            timer0 = 0;
            timer0active = false;

            prevTime = -1;
            currentTime = 0;
        }

        private class A0Auto extends ControlAutomaton {
            public static final int STATE_0 = 0;
            public static final int STATE_1 = 1;
            public static final int STATE_2 = 2;
            public static final int STATE_3 = 3;
            public static final int STATE_4 = 4;

            public static final int VOLTAGE_TIME = 2000;
            public static final int ES_POWER = 1;

            private boolean transportListener = false;

            private int itemsNumber = 0;

            public A0Auto(int id) {
                super(id);
                initialState = STATE_0;
                init();
            }

            public void init() {
                super.init();
                transportListener = false;
                itemsNumber = 0;
            }

            //----------------------------------------------------------------------------------------------------------
            private boolean x0_0() {
                return (touchSensor.getValue() == TouchSensor.TOUCHED_STATE);
            }

            private boolean x0_1() {
                return itemsNumber > 0;
            }

            private boolean x0_2() {
                if (transportListener) {
                    return irdaModule.isMessageFromRobotReceived();
                } else {
                    while(irdaModule.isMessageFromRobotReceived()) {
                        irdaModule.readMessage();
                    }

                    return false;
                }
            }

            private boolean x0_3() {
                return (timer0active && (timer0 > VOLTAGE_TIME));
            }
            //----------------------------------------------------------------------------------------------------------
            private void z0_0() {
                lcdMessage = "2.0V";
            }

            private void z0_1() {
                startDispencing(ES_POWER);
            }

            private void z0_2() {
                stopDispencing();
            }

            private void z0_3() {
                itemsNumber--;
            }

            private void z0_4() {
                irdaModule.sendMessage((byte) (128 + 1));
            }


            private void z0_5_0() {
                transportListener = false;
                while(irdaModule.isMessageFromRobotReceived()) {
                    irdaModule.readMessage();
                }
            }

            private void z0_5_1() {
                transportListener = true;
                while(irdaModule.isMessageFromRobotReceived()) {
                    irdaModule.readMessage();
                }
            }

            private void z0_5_2() {
                if (irdaModule.isMessageFromRobotReceived()) {
                    itemsNumber = irdaModule.readMessage();
                }
            }

            private void z0_6() {
                timer0active = true;
                timer0 = 0;
            }

            private void z0_7() {
                timer0active = false;
                timer0 = 0;
            }

            //----------------------------------------------------------------------------------------------------------
            public synchronized void updateAutoState(int event, int data) {
                int oldState = getAutoState();

                switch(oldState) {
                    case STATE_0: {
                        //----------------------------------------------------------------------------------------------

                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        setAutoState(STATE_4);
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_0();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------

                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_1: {
                        //----------------------------------------------------------------------------------------------
                        boolean x0_2 = x0_2();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x0_2) {
                            setAutoState(STATE_2);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_2) {
                            z0_5_2();
                            z0_5_0();
                            z0_1();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_2: {
                        //----------------------------------------------------------------------------------------------
                        boolean x0_0  = x0_0();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (!x0_0) {
                            setAutoState(STATE_3);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_3: {
                        //----------------------------------------------------------------------------------------------
                        boolean x0_0 = x0_0();
                        boolean x0_1 = x0_1();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (!x0_1) {
                            setAutoState(STATE_1);
                        } else

                        if (x0_0 && x0_1) {
                            setAutoState(STATE_2);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged =  true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_1) {
                            z0_2();
                            z0_4();
                            z0_5_1();
                        } else

                        if (getAutoState() == STATE_2) {
                            z0_3();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_4: {
                        //----------------------------------------------------------------------------------------------
                        boolean x0_3 = x0_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_6();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x0_3) {
                            setAutoState(STATE_1);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_7();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_1) {
                            z0_5_1();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }
                }
            }

            public int getAutoState() {
                return state;
            }

            public void setAutoState(int state) {
                this.state = state;
            }

            //no commands here from other automata
            public void addCommand(int autoId, int commandId) {

            }
        }
    }

    public String toString() {
        String res = system.a0.toString() + "; " + dispencerHookPosition + "; " +
                ((DispencerControlSystem.A0Auto) system.a0).itemsNumber;
        return res;
    }
}