package robots.field;

import robots.transport.SixWheeledTransport;
import robots.sensors.LightSensor;
import robots.sensors.Sensor;
import robots.comm.*;
import robots.remote.RemoteControl;
import robots.dispencer.Dispencer;

import java.util.List;
import java.util.Iterator;

import physics.CalcObject;
import physics.PhysicalObject;

/**
 * Boris Yartsev
 * This is the class, where the emulation of the transport tobot is performed.
 * todo implement the auto.Automaton interface correctly
 */
public class BattleField implements CalcObject {
    private int width;
    private int height;

    private int calcState = STOP_CALC_STATE;

    //this state denotes, when the simulation state is stopped
    public static final int STOP_CALC_STATE = 1;

    //this state denotes when the simulation is performed
    public static final int CALC_CALC_STATE = 2;

    public static final int E1_START_CALC_EVENT = 1;
    public static final int E2_STOP_CALC_EVENT = 2;
    public static final int E3_PAUSE_CALC_EVENT = 4;
    public static final int E4_LOAD_DATA_CALC_EVENT = 5;

    //This is just the robot from the lego isenguard project
    private SixWheeledTransport transport;

    //This is the dispencer robot from the project
    private Dispencer dispencer;

    //This is the remoteControl. There is now no unique interface for handling remote controls and other robots on the field
    private RemoteControl remoteControl;

    //it is the data, which is used for editing
    private BattleFieldData battleData;

    //This is used for the infraredrecievers and infrared emitters
    private InfraredField infraredField;

    public BattleField(int width, int height) {
        this.width = width;
        this.height = height;

        transport = createTransport();
        dispencer = createDispencer();
        battleData = createBattleFieldData();
        battleData.reloadCoords();

        remoteControl = createRemoteControl();

        infraredField = new InfraredField();
        infraredField.addEmitter(remoteControl.getEmitter());
        infraredField.addReceiverEmitter(transport.getIrdaModule().getReceiverEmitter());
        infraredField.addReceiverEmitter(dispencer.getIrdaModule().getReceiverEmitter());

    }

    private SixWheeledTransport createTransport() {
        return new SixWheeledTransport();
    }

    private Dispencer createDispencer() {
        return new Dispencer();
    }

    private RemoteControl createRemoteControl() {
        return new RemoteControl();
    }

    public BattleFieldData createBattleFieldData() {
        return new BattleFieldData(0, width - 1, 0, height - 1);
    }

    public SixWheeledTransport getTransport() {
        return transport;
    }

    public Dispencer getDispencer() {
        return dispencer;
    }

    public RemoteControl getRemoteControl() {
        return remoteControl;
    }

    public InfraredField getInfraredField() {
        return infraredField;
    }

    public void start() {
        updateCalcState(E1_START_CALC_EVENT, 0, 0, null);
    }

    /**
     * The syncronization is needed because the method updateCoordinates may have not stopped at this moment
     * todo move everything to one method updateCalcState or something similar
     */
    public void stop() {
        updateCalcState(E2_STOP_CALC_EVENT, 0, 0, null);
    }

    public void pause() {
        updateCalcState(E3_PAUSE_CALC_EVENT, 0, 0, null);
    }

    public void loadNewData(BattleFieldData data) {
        updateCalcState(E4_LOAD_DATA_CALC_EVENT, 0, 0, data);
    }

    public boolean isSimulation() {
        return getCalcState() == CALC_CALC_STATE;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    public class BattleFieldData {
        public static final int FIELD = 0;
        public static final int PATH = 1;
        public static final int FOIL = 2;
        public static final int ERROR = 3;

        public static final int FIELD_INTENSITY = 60;
        public static final int PATH_INTENSITY = 40;
        public static final int FOIL_INTENSITY = 80;
        public static final int ERROR_INTENSITY = 0;

        //The starting coordinates of the transport robot
        private double xStart = 100;
        private double yStart = 100;
        private double angleStart = 0;

        //the starting coordinates of the dispencer robot
        private double xDis = 200;
        private double yDis = 100;
        private double angleDis = 0;

        private byte [][] points;

        private int xmin;
        private int xmax;
        private int ymin;
        private int ymax;

        private BattleFieldData(int xmin, int xmax, int ymin, int ymax) {
            this.xmin = xmin;
            this.xmax = xmax;
            this.ymin = ymin;
            this.ymax = ymax;

            points = new byte[xmax - xmin + 1][ymax - ymin + 1]; //filled with the default values of emply fields
        }

        public boolean compareSize(BattleFieldData data) {
            boolean res = data.xmax == xmax;
            res = res && data.xmin == xmin;
            res = res && data.ymax == ymax;
            res = res && data.ymin == ymin;
            return res;
        }

        public void setPointAt(int x, int y, int type) {
            points[x - xmin][y - ymin] = (byte) type;
        }

        public int getPointAt(int x, int y) {
            if (x <= xmax && x >= xmin && y <= ymax && y >= ymin) {
                return points[x - xmin][y - ymin];
            } else {
                return BattleFieldData.ERROR;
            }
        }

        public double getTransportX() {
            return xStart;
        }

        public double getTransportY() {
            return yStart;
        }

        public double getTransportAngle() {
            return angleStart;
        }

        public double getDispencerX() {
            return xDis;
        }

        public double getDispencerY() {
            return yDis;
        }

        public double getDispencerAngle() {
            return angleDis;
        }

        public void setTransportCoords(double x, double y, double angle) {
            xStart = x;
            yStart = y;
            angleStart = angle;
        }

        public void setDispencerCoords(double x, double y, double angle) {
            xDis = x;
            yDis = y;
            angleDis = angle;
        }

        private void reloadCoords() {
            transport.setX(xStart);
            transport.setY(yStart);
            transport.setAngle(angleStart);

            dispencer.setX(xDis);
            dispencer.setY(yDis);
            dispencer.setAngle(angleDis);
        }
    }

    public synchronized void updateCalcState(int calcEvent, int data, double delta, Object objData) {
        switch(getCalcState())  {
            case STOP_CALC_STATE:
                switch(calcEvent) {
                    case E1_START_CALC_EVENT:
                        z2_restartObjects();
                        z0_reloadCoords();
                        setCalcState(CALC_CALC_STATE);
                        break;

                    case E2_STOP_CALC_EVENT:
                        z2_restartObjects();
                        z0_reloadCoords();
                        break;

                    case E3_PAUSE_CALC_EVENT:
                        setCalcState(CALC_CALC_STATE);
                        break;

                    case E4_LOAD_DATA_CALC_EVENT:
                        z2_restartObjects();
                        z1_loadNewData((BattleFieldData) objData);
                        break;
                }

                break;

            case CALC_CALC_STATE:
                switch(calcEvent) {
                    case E1_START_CALC_EVENT:
                        z2_restartObjects();
                        z0_reloadCoords();
                        break;

                    case E2_STOP_CALC_EVENT:
                        z2_restartObjects();
                        z0_reloadCoords();
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E3_PAUSE_CALC_EVENT:
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E4_LOAD_DATA_CALC_EVENT:
                        z2_restartObjects();
                        z1_loadNewData((BattleFieldData) objData);
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        z4_updateCoordinates(delta);
                        break;
                }

                break;
        }
    }

    private void z0_reloadCoords() {
        battleData.reloadCoords();
    }

    private void z1_loadNewData(BattleFieldData fieldData) {
        /**
         * If the current battleFieldData has the same size that the fieldData, being loaded, then the loading is performed
         */
        if (battleData.compareSize(fieldData)) {
            for (int i = battleData.xmin; i <= battleData.xmax; i++) {
                for (int j = battleData.ymin; j <= battleData.ymax; j++) {
                    battleData.setPointAt(i, j, fieldData.getPointAt(i, j));
                }
            }

            battleData.setTransportCoords(fieldData.xStart, fieldData.yStart, fieldData.angleStart);
            battleData.setDispencerCoords(fieldData.xDis, fieldData.yDis, fieldData.angleDis);
            battleData.reloadCoords();
        }
    }

    private void z2_restartObjects() {
        transport.restart();
        dispencer.restart();
    }

    //todo make the sensor calcuate the average value from several points
    //todo now it takes only one point and uses it as the source for the light sensor
    public void z4_updateCoordinates(double timeDelta) {
        //Updating the state of the robots on the field
        transport.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
        dispencer.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
        infraredField.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);

        List sensorList = transport.getSensors();

        for (Iterator iter = sensorList.iterator(); iter.hasNext(); ) {
            Sensor sensor = (Sensor) iter.next();
            if (sensor instanceof LightSensor) {
                LightSensor ls = (LightSensor) sensor;

                int type = battleData.getPointAt((int) ls.getX(), (int) ls.getY());

                int intensity = 0;
                switch(type) {
                    case BattleFieldData.FIELD:
                        intensity = BattleFieldData.FIELD_INTENSITY;
                        break;

                    case BattleFieldData.FOIL:
                        intensity = BattleFieldData.FOIL_INTENSITY;
                        break;

                    case BattleFieldData.PATH:
                        intensity = BattleFieldData.PATH_INTENSITY;
                        break;

                    case BattleFieldData.ERROR:
                        intensity = BattleFieldData.ERROR_INTENSITY;
                        break;
                }

                ls.intensityChanged(intensity);
            }
        }

        //taking the item from the conveyor
        Dispencer.Conveyor conveyor = dispencer.getConveyor();

        if (conveyor.isItemReady()) {
            if (canDispence(dispencer.getX(), dispencer.getY(), dispencer.getAngle(),
                    transport.getX(), transport.getY(), transport.getAngle())) {
                transport.loadItem(conveyor.getItemReady());
            }
        }
    }

    private double dispenceRadius = 100;
    private boolean canDispence(double disX, double disY, double disAngle, double trX, double trY, double trAngle) {
        double dx = disX - trX;
        double dy = disY - trY;
        double sqr = dx * dx + dy * dy;

        return sqr < (dispenceRadius * dispenceRadius);
    }

    public int getCalcState() {
        return calcState;
    }

    public void setCalcState(int calcState) {
        this.calcState = calcState;
    }
}
