package robots.transport;

import physics.PhysicalObject;
import physics.Constants;
import physics.CalcObject;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.*;
import java.util.List;

import robots.sensors.TouchSensor;
import robots.sensors.LightSensor;
import robots.comm.RobotIRDAModule;
import robots.view.MandMFactory;
import robots.FieldRobot;
import robots.ControlSystem;
import auto.ControlAutomaton;

/**
 * This is the six wheeled robot with which the emulation will be performed
 * Boris Yartsev
 */
public class SixWheeledTransport extends PhysicalObject implements FieldRobot {
    public static final int DRIVE_FORWARD_CALC_STATE = 1;
    public static final int DRIVE_BACKWARD_CALC_STATE = 2;
    public static final int TURN_LEFT_CALC_STATE = 3;
    public static final int TURN_RIGHT_CALC_STATE = 4;
    public static final int STOP_CALC_STATE = 5;

    private double speedPerPowerUnit = 5;
    private double torquePerPowerUnit = 0.05;

    private double speedAcc = 20;
    private double torqueAcc = 0.1;

    private double speed;

    private List sensors;

    private LightSensor fixedLightSensor;
    private double fixedLightRadius = 20;
    private double fixedLightAngle = 0;

    private LightSensor rodLightSensor;
    private Rod rod;
    private double rodRadius = 25;
    private double rodAngle = 0.0;

    private TouchSensor touchSensor;

    private RobotIRDAModule irdaModule;
    private double irdaModuleRadius = 15;
    private double irdaModuleAngle = 0;

    public static final double sensorTouchAngle = Math.PI / 36;

    public static final int MAX_POWER = 7;

    private int power = 0;

    private LineTraceControlSystem system;

    private String transportImageLocation = "images\\robots\\transport.gif";

    private Image transportImage = Toolkit.getDefaultToolkit().createImage(transportImageLocation);
    private Image transportBufferImage = null;

    private boolean shown = true;

    //the real time of the system
    private double time;

    //maximum load capacity of transport
    public static final int LOAD_CAPACITY = 3;

    //the number of items the transport is delivering
    private int deliveryNumber = 0;

    private int items[];

    private int xCoord[] = {
        5, 12, 5
    };

    private int yCoord[] = {
        11, 17, 22
    };

    private String description = "Transport robot";

    private String lcdMessage = "";

    private Image greenImage = MandMFactory.createMandM(MandMFactory.MANDM_GREEN);
    private Image redImage = MandMFactory.createMandM(MandMFactory.MANDM_RED);
    private Image yellowImage = MandMFactory.createMandM(MandMFactory.MANDM_YELLOW);
    private Image blueImage = MandMFactory.createMandM(MandMFactory.MANDM_BLUE);

    public SixWheeledTransport() {
        super(0, 0, 0);
        rod = new Rod(rodRadius, rodAngle);

        sensors = new ArrayList();

        fixedLightSensor = new LightSensor("Fixed light sensor");
        sensors.add(fixedLightSensor);

        rodLightSensor = new LightSensor("Rod light sensor");
        sensors.add(rodLightSensor);

        touchSensor = new TouchSensor("Rod touch sensor");
        sensors.add(touchSensor);

        irdaModule = new RobotIRDAModule();
        irdaModule.getReceiverEmitter().setShown(false);

        //there should be only one instance of the control system
        system = new LineTraceControlSystem();

        items = new int[LOAD_CAPACITY];

        setCalcState(STOP_CALC_STATE);
    }

    //This weird procedure is added to make the setting of coordinates for different objects, that surround transport robot
    public void setX(double x) {
        super.setX(x);
        setAngle(getAngle());
    }

    public void setY(double y) {
        super.setY(y);
        setAngle(getAngle());
    }

    public void setAngle(double angle) {
        super.setAngle(angle);
        updateSensorsCoords();
        updateIrdaCoords();
    }

    private void updateSensorsCoords() {
        fixedLightSensor.setX(x + Math.cos(angle + fixedLightAngle) * fixedLightRadius);
        fixedLightSensor.setY(y + Math.sin(angle + fixedLightAngle) * fixedLightRadius);

        rodLightSensor.setX(x + Math.cos(angle + rod.getAngle()) * rodRadius);
        rodLightSensor.setY(y + Math.sin(angle + rod.getAngle()) * rodRadius);

        if (Math.abs(rod.getAngle() - rod.getInitialAngle()) % (Math.PI * 2) < sensorTouchAngle) {
            touchSensor.touch();
        } else {
            touchSensor.untouch();
        }
    }

    private void updateIrdaCoords() {
        irdaModule.setX(x + irdaModuleRadius * Math.cos(angle + irdaModuleAngle));
        irdaModule.setY(y + irdaModuleRadius * Math.sin(angle + irdaModuleAngle));
        irdaModule.setAngle(angle + irdaModuleAngle);
    }

    private void z0_setPower(int power) {
        if (power > MAX_POWER) {
            power = MAX_POWER;
        }

        if (power < 0) {
            power = 0;
        }

        this.power = power;
    }

    //here the speed and the sensors are reset
    private void z1_restart() {
        touchSensor.untouch();
        fixedLightSensor.deactivate();
        rodLightSensor.deactivate();

        lcdMessage = "";
        speed = 0;
        wTorque = 0;
        deliveryNumber = 0;
        time = 0;
    }

    private void z2_updateTime(double timeDelta) {
        time += timeDelta;
    }

    private void z3_1_updateCoordinatesForward(double timeDelta) {
        if (Math.abs(wTorque) > 0) {
            double signum = Math.signum(wTorque);
            wTorque += -signum * torqueAcc * timeDelta;
            double newSignum = Math.signum(wTorque);

            if (newSignum * signum < 0) {
                wTorque = 0;
            }
        }

        if (speed < speedPerPowerUnit * power) {
            speed += speedAcc * timeDelta;
        } else {
            speed -= speedAcc * timeDelta;
        }
    }

    private void z3_2_updateCoordinatesBackward(double timeDelta) {
        if (Math.abs(wTorque) > 0) {
            double signum = Math.signum(wTorque);
            wTorque += -signum * torqueAcc * timeDelta;
            double newSignum = Math.signum(wTorque);

            if (newSignum * signum < 0) {
                wTorque = 0;
            }
        }

        if (speed > -speedPerPowerUnit * power) {
            speed -= speedAcc * timeDelta;
        } else {
            speed += speedAcc * timeDelta;
        }
    }

    private void z3_3_updateCoordinatesLeft(double timeDelta) {
        if (Math.abs(speed) > Constants.EPS) {
            double signum = Math.signum(speed);
            speed += -signum * speedAcc * timeDelta;
            double newSignum = Math.signum(speed);

            if (signum * newSignum < 0) {
                speed = 0;
            }
        }

        if (wTorque < torquePerPowerUnit * power) {
            wTorque += torqueAcc * timeDelta;
        } else {
            wTorque -= torqueAcc * timeDelta;
        }
    }

    private void z3_4_updateCoordinatesRight(double timeDelta) {
        if (Math.abs(speed) > Constants.EPS) {
            double signum = Math.signum(speed);
            speed += -signum * speedAcc * timeDelta;
            double newSignum = Math.signum(speed);

            if (signum * newSignum < 0) {
                speed = 0;
            }
        }

        if (wTorque > -torquePerPowerUnit * power) {
            wTorque -= torqueAcc * timeDelta;
        } else {
            wTorque += torqueAcc * timeDelta;
        }
    }

    private void z3_5_updateCoordinatesStop(double timeDelta) {
        if (Math.abs(speed) > Constants.EPS) {
            double signum = Math.signum(speed);
            speed += -signum * speedAcc * timeDelta;
            double newSignum = Math.signum(speed);

            if (signum * newSignum < 0) {
                speed = 0;
            }
        }

        if (Math.abs(wTorque) > 0) {
            double signum = Math.signum(wTorque);
            wTorque += -signum * torqueAcc * timeDelta;
            double newSignum = Math.signum(wTorque);

            if (newSignum * signum < 0) {
                wTorque = 0;
            }
        }
    }

    private void z3_6_updateCoordinates(double timeDelta) {
        setX(getX() + speed * Math.cos(getAngle()) * timeDelta);
        setY(getY() + speed * Math.sin(getAngle()) * timeDelta);

        //update sensors coords is performed in the setAngle method
        setAngle(getAngle() + wTorque * timeDelta);
    }

    public void setPower(int power) {
        updateCalcState(E6_UPDATE_POWER_CALC_EVENT, power, 0, null);
    }

    public int getPower() {
        return power;
    }

    public RobotIRDAModule getIrdaModule() {
        return irdaModule;
    }

    public static final int E1_MOVE_FORWARD_CALC_EVENT = 1;
    public static final int E2_MOVE_BACKWARD_CALC_EVENT = 2;
    public static final int E3_MOVE_LEFT_CALC_EVENT = 3;
    public static final int E4_MOVE_RIGHT_CALC_EVENT = 4;
    public static final int E5_STOP_CALC_EVENT = 5;
    public static final int E6_UPDATE_POWER_CALC_EVENT = 6;


    public void moveForward(int power) {
        updateCalcState(E1_MOVE_FORWARD_CALC_EVENT, power, 0, null) ;
    }

    public void moveBackward(int power) {
        updateCalcState(E2_MOVE_BACKWARD_CALC_EVENT, power, 0, null);
    }

    public void turnLeft(int power) {
        updateCalcState(E3_MOVE_LEFT_CALC_EVENT, power, 0, null) ;
    }

    public void turnRight(int power) {
        updateCalcState(E4_MOVE_RIGHT_CALC_EVENT, power, 0, null) ;
    }

    public void stop() {
        updateCalcState(E5_STOP_CALC_EVENT, 0, 0, null);
    }

    public void restart() {
        updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
    }

    //loads a new item to the transport - transport maximum capacity is three objects
    public void loadItem(int mandmType) {
        if (deliveryNumber < LOAD_CAPACITY) {
            items[deliveryNumber] = mandmType;
            deliveryNumber++;
        }
    }

    public void unloadAllItems() {
        deliveryNumber = 0;
    }

    public synchronized void updateCalcState(int event, int data, double timeDelta, Object objData) {
        switch(getCalcState()) {
            case DRIVE_FORWARD_CALC_STATE: {
                switch(event) {
                    case E1_MOVE_FORWARD_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case E2_MOVE_BACKWARD_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(DRIVE_BACKWARD_CALC_STATE);
                        break;

                    case E3_MOVE_LEFT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_LEFT_CALC_STATE);
                        break;

                    case E4_MOVE_RIGHT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_RIGHT_CALC_STATE);
                        break;

                    case E5_STOP_CALC_EVENT:
                        //z0_setPower(data);
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E6_UPDATE_POWER_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        irdaModule.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        system.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);

                        z2_updateTime(timeDelta);
                        z3_1_updateCoordinatesForward(timeDelta);
                        z3_6_updateCoordinates(timeDelta);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
                        irdaModule.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT,  0, 0, null);
                        system.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);

                        z1_restart();
                        setCalcState(STOP_CALC_STATE);
                        break;
                }

                break;
            }
            case DRIVE_BACKWARD_CALC_STATE: {
                switch(event) {
                    case E1_MOVE_FORWARD_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(DRIVE_FORWARD_CALC_STATE);
                        break;

                    case E2_MOVE_BACKWARD_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case E3_MOVE_LEFT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_LEFT_CALC_STATE);
                        break;

                    case E4_MOVE_RIGHT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_RIGHT_CALC_STATE);
                        break;

                    case E5_STOP_CALC_EVENT:
                        //z0_setPower(data);
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E6_UPDATE_POWER_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        irdaModule.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        system.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);

                        z2_updateTime(timeDelta);
                        z3_2_updateCoordinatesBackward(timeDelta);
                        z3_6_updateCoordinates(timeDelta);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
                        irdaModule.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT,  0, 0, null);
                        system.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);

                        z1_restart();
                        setCalcState(STOP_CALC_STATE);
                        break;
                }
                break;
            }
            case TURN_LEFT_CALC_STATE: {
                switch(event) {
                    case E1_MOVE_FORWARD_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(DRIVE_FORWARD_CALC_STATE);
                        break;

                    case E2_MOVE_BACKWARD_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(DRIVE_BACKWARD_CALC_STATE);
                        break;

                    case E3_MOVE_LEFT_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case E4_MOVE_RIGHT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_RIGHT_CALC_STATE);
                        break;

                    case E5_STOP_CALC_EVENT:
                        //z0_setPower(data);
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E6_UPDATE_POWER_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        irdaModule.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        system.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);

                        z2_updateTime(timeDelta);
                        z3_3_updateCoordinatesLeft(timeDelta);
                        z3_6_updateCoordinates(timeDelta);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
                        irdaModule.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT,  0, 0, null);
                        system.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);

                        z1_restart();
                        setCalcState(STOP_CALC_STATE);
                        break;
                }
                break;
            }
            case TURN_RIGHT_CALC_STATE: {
                switch(event) {
                    case E1_MOVE_FORWARD_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(DRIVE_FORWARD_CALC_STATE);
                        break;

                    case E2_MOVE_BACKWARD_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(DRIVE_BACKWARD_CALC_STATE);
                        break;

                    case E3_MOVE_LEFT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_LEFT_CALC_STATE);
                        break;

                    case E4_MOVE_RIGHT_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case E5_STOP_CALC_EVENT:
                        //z0_setPower(data);
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E6_UPDATE_POWER_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        irdaModule.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        system.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);

                        z2_updateTime(timeDelta);
                        z3_4_updateCoordinatesRight(timeDelta);
                        z3_6_updateCoordinates(timeDelta);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
                        irdaModule.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT,  0, 0, null);
                        system.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);

                        z1_restart();
                        setCalcState(STOP_CALC_STATE);
                        break;
                }
                break;
            }
            case STOP_CALC_STATE: {
                switch(event) {
                    case E1_MOVE_FORWARD_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(DRIVE_FORWARD_CALC_STATE);
                        break;

                    case E2_MOVE_BACKWARD_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(DRIVE_BACKWARD_CALC_STATE);
                        break;

                    case E3_MOVE_LEFT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_LEFT_CALC_STATE);
                        break;

                    case E4_MOVE_RIGHT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_RIGHT_CALC_STATE);
                        break;
                    /*
                    case E5_STOP_CALC_EVENT:
                        //z0_setPower(data);
                        break;
                    */
                    case E6_UPDATE_POWER_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        irdaModule.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
                        system.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);

                        z2_updateTime(timeDelta);
                        z3_5_updateCoordinatesStop(timeDelta);
                        z3_6_updateCoordinates(timeDelta);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        rod.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
                        irdaModule.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT,  0, 0, null);
                        system.updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);

                        z1_restart();
                        setCalcState(STOP_CALC_STATE);
                        break;
                }
                break;
            }
        }
    }

    public List getSensors() {
        return sensors;
    }

    public ControlSystem getControlSystem() {
        return system;
    }

    public String getDescription() {
        return description;
    }

    public String getLcdMessage() {
        return lcdMessage;
    }

    public Image getObjectImage() {
        if (!(transportImage.getWidth(null) > 0 && transportImage.getHeight(null) > 0)) {
            return null;
        }

        if (transportBufferImage == null) {
            transportBufferImage = new BufferedImage(transportImage.getWidth(null),
                    transportImage.getHeight(null), BufferedImage.TYPE_INT_ARGB);
        }

        Graphics bg = transportBufferImage.getGraphics();
        bg.drawImage(transportImage, 0, 0, null);

        for (int i = 0; i < deliveryNumber; i++) {
            Image img = null;
            switch(items[i]) {
                case MandMFactory.MANDM_GREEN:
                    img = greenImage;
                    break;

                case MandMFactory.MANDM_BLUE:
                    img = blueImage;
                    break;

                case MandMFactory.MANDM_RED:
                    img = redImage;
                    break;

                case MandMFactory.MANDM_YELLOW:
                    img = yellowImage;
                    break;
            }

            if (img != null) {
                bg.drawImage(img, xCoord[i] - img.getWidth(null) / 2,
                        yCoord[i] - img.getHeight(null) / 2, null);
            }
        }

        return transportBufferImage;
    }


    public void setObjectImage(Image img) {
    }

    public boolean isShown() {
        return shown;
    }

    public void setShown(boolean shown) {
        this.shown = shown;
    }

    public class LineTraceControlSystem implements CalcObject, ControlSystem {
        private int calcState = CONTROL_CALC_STATE;

        public static final int CONTROL_CALC_STATE = 0;

        private double timer1 = 0;
        private double timer2 = 0;
        private double timer3 = 0;
        private double timer4 = 0;
        private boolean timer1active = false;
        private boolean timer2active = false;
        private boolean timer3active = false;
        private boolean timer4active = false;

        public static final int A1_ID = 1;
        public static final int A2_ID = 2;
        public static final int A3_ID = 3;
        public static final int A5_ID = 5;

        public static final int WHITE_DOWN = 42;
        public static final int WHITE_UP = 63;
        public static final int BLACK_DOWN = 20;
        public static final int BLACK_UP = 41;
        public static final int FOIL_DOWN = 64;
        public static final int FOIL_UP = 80;

        public static final int ES_POWER = 2;
        public static final int ER_POWER = 4;

        private List controlAutomata;

        private boolean x0_4_0() {
            int value = fixedLightSensor.getValue();
            return ((value >= WHITE_DOWN) && value <= WHITE_UP);
        }

        private boolean x0_4_1() {
            int value = fixedLightSensor.getValue();
            return ((value >= BLACK_DOWN) && value <= BLACK_UP);
        }

        private boolean x0_4_2() {
            int value = fixedLightSensor.getValue();
            return ((value >= FOIL_DOWN) && value <= FOIL_UP);
        }

        private boolean x0_4_3() {
            int value = fixedLightSensor.getValue();
            return ((value > FOIL_UP) || (value < BLACK_DOWN));
        }


        private boolean x0_5_0() {
            int value = rodLightSensor.getValue();
            return ((value >= WHITE_DOWN) && value <= WHITE_UP);
        }

        private boolean x0_5_1() {
            int value = rodLightSensor.getValue();
            return ((value >= BLACK_DOWN) && value <= BLACK_UP);
        }

        private boolean x0_5_2() {
            int value = rodLightSensor.getValue();
            return ((value >= FOIL_DOWN) && value <= FOIL_UP);
        }

        private boolean x0_5_3() {
            int value = rodLightSensor.getValue();
            return ((value > FOIL_UP) || (value < BLACK_DOWN));
        }

        private void z0_5_1() {
            timer1active = true;
            timer1 = 0;
        }

        private void z0_6_1() {
            timer1active = false;
            timer1 = 0;
        }

        private void z0_5_2() {
            timer2active = true;
            timer2 = 0;
        }

        private void z0_6_2() {
            timer2active = false;
            timer2 = 0;
        }

        private void z0_5_3() {
            timer3active = true;
            timer3 = 0;
        }

        private void z0_6_3() {
            timer3active = false;
            timer3 = 0;
        }

        private void z0_5_4() {
            timer4active = true;
            timer4 = 0;
        }

        private void z0_6_4() {
            timer4active = false;
            timer4 = 0;
        }

        private void z0_7() {
            moveForward(ES_POWER);
        }

        private void z0_8() {
            moveBackward(ES_POWER);
        }

        private void z0_9() {
            stop();
        }

        private void z0_10() {
            turnLeft(ER_POWER);
        }

        private void z0_11() {
            turnRight(ER_POWER);
        }

        public LineTraceControlSystem() {
            controlAutomata = new ArrayList();
            a1 = new A1Auto(A1_ID);
            a2 = new A2Auto(A2_ID);
            a3 = new A3Auto(A3_ID);
            a5 = new A5Auto(A5_ID);

            controlAutomata.add(a1);
            controlAutomata.add(a2);
            controlAutomata.add(a3);
            controlAutomata.add(a5);
        }

        private ControlAutomaton a1;
        private ControlAutomaton a2;
        private ControlAutomaton a3;
        private ControlAutomaton a5;

        public List getControlAutomata() {
            return controlAutomata;
        }

        public void restart() {
            updateCalcState(PhysicalObject.E101_RESTART_CALC_EVENT, 0, 0, null);
        }

        /**
         * Here data is the time in the milliseconds
         * @param event
         * @param data
         * @param delta
         * @param obj
         */
        public void updateCalcState(int event, int data, double delta, Object obj) {
            switch (getCalcState()) {
                case CONTROL_CALC_STATE: {
                    switch(event) {
                        case PhysicalObject.E101_RESTART_CALC_EVENT:
                            z3_restartControlSystem();
                            break;

                        case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                            z0_updateTimers(delta);
                            z2_updateControlSystemState();
                            break;
                    }
                    break;
                }
            }
        }

        private long prevTime = -1;
        private double currentTime = 0;
        //private long

        private void z0_updateTimers(double timeDouble) {
            currentTime += timeDouble;

            long time = (long) (1000 * currentTime);

            if (prevTime == -1) {
                prevTime = time;
            }

            if (timer1active) {
                timer1 += time - prevTime;
            }

            if (timer2active) {
                timer2 += time - prevTime;
            }

            if (timer3active) {
                timer3 += time - prevTime;
            }

            if (timer4active) {
                timer4 += time - prevTime;
            }

            prevTime = time;
        }

        private void z2_updateControlSystemState() {
            a1.updateAutoState(0, 0);
        }

        private void z3_restartControlSystem() {
            a1.init();
            a2.init();
            a3.init();
            a5.init();
            timer1 = 0;
            timer1active = false;
            timer2 = 0;
            timer2active = false;
            timer3 = 0;
            timer3active = false;
            timer4 = 0;
            timer4active = false;

            prevTime = -1;
            currentTime = 0;
        }

        public int getCalcState() {
            return calcState;
        }

        public void setCalcState(int state) {
            this.calcState = state;
        }

        private class A1Auto extends ControlAutomaton {
            public static final int STATE_0 = 0;
            public static final int STATE_1 = 1;
            public static final int STATE_2 = 2;
            public static final int STATE_3 = 3;
            public static final int STATE_4 = 4;
            public static final int STATE_5 = 5;
            public static final int STATE_6 = 6;
            public static final int STATE_7 = 7;
            public static final int STATE_8 = 8;
            public static final int STATE_9 = 9;
            public static final int STATE_10 = 10;

            public static final int A1_TRACE_PATH_COMMAND = 1;

            public static final int TURN_TIME = 16000;
            public static final int BACK_TIME = 3000;
            public static final int VOLTAGE_TIME = 2000;

            private boolean A1_A2_Listener = false;
            private boolean A1_PathFinished = false;

            private boolean remoteListener = false;
            private boolean robotListener = false;

            private int itemsNumber = 0;

            public A1Auto(int id) {
                super(id);
                initialState = STATE_0;
                init();
            }

            public void init() {
                super.init();
                A1_A2_Listener = false;
                A1_PathFinished = false;
                itemsNumber = 0;
            }

            private boolean x1_0_0() {
                if (remoteListener) {
                    return irdaModule.isMessageFromRemoteReceived();
                } else {
                    while(irdaModule.isMessageFromRemoteReceived()) {
                        irdaModule.readMessage();
                    }

                    return false;
                }
            }

            private boolean x1_0_1() {
                if (robotListener) {
                    return irdaModule.isMessageFromRobotReceived();
                } else {
                    while(irdaModule.isMessageFromRobotReceived()) {
                        irdaModule.readMessage();
                    }

                    return false;
                }
            }

            private boolean x1_2() {
                return (timer1active && (timer1 > TURN_TIME));
            }

            private boolean x1_3() {
                return (timer1active && (timer1 > BACK_TIME));
            }

            private boolean x1_4() {
                return (timer1active && (timer1 > VOLTAGE_TIME));
            }

            private boolean x1_6() {
                return A1_PathFinished;
            }

            //----------------------------------------------------------------------------------------------------------
            private void z1_0() {
                lcdMessage = "2.0V";
            }

            //fixed light sensor activation
            private void z1_1() {
                fixedLightSensor.activate();
            }

            //rod light sensor activation
            private void z1_2() {
                rodLightSensor.activate();
            }

            private void z1_3_0() {
                remoteListener = false;
                while(irdaModule.isMessageFromRemoteReceived()) {
                    irdaModule.readMessage();
                }
            }

            private void z1_3_1() {
                remoteListener = true;
                while(irdaModule.isMessageFromRemoteReceived()) {
                    irdaModule.readMessage();
                }
            }

            private void z1_3_2() {
                if (irdaModule.isMessageFromRemoteReceived()) {
                    itemsNumber = irdaModule.readMessage();
                }
            }

            private void z1_4_0() {
                robotListener = false;
                while(irdaModule.isMessageFromRobotReceived()) {
                    irdaModule.readMessage();
                }
            }

            private void z1_4_1() {
                robotListener = true;
                while(irdaModule.isMessageFromRobotReceived()) {
                    irdaModule.readMessage();
                }
            }

            private void z1_4_2() {
                if (irdaModule.isMessageFromRobotReceived()) {
                    irdaModule.readMessage();
                }
            }

            private void z1_5() {
                lcdMessage = "ERROR!!!";
            }

            private void z1_6() {
                irdaModule.sendMessage((byte) (128 + itemsNumber));
            }

            private void z1_7_0() {
                A1_A2_Listener = false;
            }

            private void z1_7_1() {
                A1_A2_Listener = true;
            }

            private void z1_7_2() {
                A1_PathFinished = false;
            }

            private void z1_8() {
                a2.addCommand(getId(), A1_TRACE_PATH_COMMAND);
            }

            private void z9() {
                deliveryNumber = 0;
            }

            //----------------------------------------------------------------------------------------------------------
            public void updateAutoState(int event, int data) {
                int oldState = getAutoState();

                switch(oldState) {
                    case STATE_0: {
                        //----------------------------------------------------------------------------------------------

                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        setAutoState(STATE_1);
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z1_0();
                            z1_1();
                            z1_2();
                            z1_4_0();
                            z1_3_1();
                            z1_7_0();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------

                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_1: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_4 = x1_4();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_1();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_4) {
                            setAutoState(STATE_2);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_1();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_2: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_0_0 = x1_0_0();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_0_0) {
                            setAutoState(STATE_3);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_3) {
                            z1_3_2();
                            z1_3_0();
                            z1_4_1();

                            //todo VIP this outputAction is added!!!
                            z9();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_3: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_2 = x1_2();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_1();
                            z0_11();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_2) {
                            setAutoState(STATE_4);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_1();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_4) {
                            z0_9();
                            z1_7_1();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_4: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_6 = x1_6();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z1_8();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        a2.updateAutoState(event, data);
                        //----------------------------------------------------------------------------------------------
                        if (a2.getAutoState() == A2Auto.STATE_3) {
                            state = STATE_10;
                        } else

                        if (x1_6) {
                            state = STATE_5;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z1_7_2();
                            z1_7_0();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_5) {
                            z0_7();
                        } else

                        if (getAutoState() == STATE_10) {
                            z1_5();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_5: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_3 = x1_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_1();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_3) {
                            setAutoState(STATE_6);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_1();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_6) {
                            z0_9();
                            z1_6();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_6: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_0_1 = x1_0_1();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_0_1) {
                            setAutoState(STATE_7);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_7) {
                            z1_4_2();
                            z1_4_0();
                            z1_3_1();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_7: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_3 = x1_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_8();
                            z0_5_1();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_3) {
                            setAutoState(STATE_8);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_1();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_8) {
                            z0_9();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_8: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_2 = x1_2();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_1();
                            z0_11();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_2) {
                            setAutoState(STATE_9);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_1();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_9) {
                            z0_9();
                            z1_7_1();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_9: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_6 = x1_6();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z1_8();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        a2.updateAutoState(event, data);
                        //----------------------------------------------------------------------------------------------
                        if (a2.getAutoState() == A2Auto.STATE_3) {
                            setAutoState(STATE_10);
                        } else

                        if (x1_6) {
                            setAutoState(STATE_2);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z1_7_2();
                            z1_7_0();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_10) {
                            z1_5();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_10: {
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }
                }
            }

            /*
            public void updateAutoState(int event, int data) {
                int oldState = getAutoState();

                switch(oldState) {
                    case STATE_0: {
                        //----------------------------------------------------------------------------------------------

                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        state = STATE_7;
                        //----------------------------------------------------------------------------------------------
                        if (oldState != state) {
                            z1_0();
                            z1_1();
                            z1_2();
                            z1_4_0();
                            z1_3_1();
                            z1_7_0();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------

                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_7: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_4 = x1_4();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_1();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_4) {
                            state = STATE_1;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != state) {
                            z0_6_1();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_1: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_0_0 = x1_0_0();
                        boolean x1_0_1 = x1_0_1();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_0_0) {
                            state = STATE_2;
                        } else

                        if (x1_0_1) {
                            state = STATE_4;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != state) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (state == STATE_2) {
                            z1_3_2();
                            z1_3_0();
                            z1_4_1();

                            //todo VIP this outputAction is added!!!
                            z9();
                        } else

                        if (state == STATE_4) {
                            z1_4_2();
                            z1_4_0();
                            z1_3_1();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_2: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_2 = x1_2();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_1();
                            z0_11();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_2) {
                            state = STATE_3;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != state) {
                            z0_6_1();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (state == STATE_3) {
                            z0_9();
                            z1_7_1();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_3: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_6 = x1_6();
                        boolean x1_8 = x1_8();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z1_8();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        a2.updateAutoState(event, data);
                        //----------------------------------------------------------------------------------------------
                        if (a2.getAutoState() == A2Auto.STATE_3) {
                            state = STATE_5;
                        } else

                        if (x1_6 && x1_8) {
                            state = STATE_6;
                        } else

                        if (x1_6 && !x1_8) {
                            state = STATE_1;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != state) {
                            z1_7_2();
                            z1_7_0();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (state == STATE_6) {
                            z0_7();
                        } else

                        if (state == STATE_5) {
                            z1_5();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_4: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_3 = x1_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_8();
                            z0_5_1();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_3) {
                            state = STATE_2;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != state) {
                            z0_6_1();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (state == STATE_2) {
                            z0_9();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_5: {
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_6: {
                        //----------------------------------------------------------------------------------------------
                        boolean x1_3 = x1_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_1();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x1_3) {
                            state = STATE_1;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != state) {
                            z0_6_1();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (state == STATE_1) {
                            z0_9();
                            z1_6();
                        }
                        //----------------------------------------------------------------------------------------------
                    }
                }
            }
            */

            public int getAutoState() {
                return state;
            }

            public void setAutoState(int state) {
                this.state = state;
            }

            public void addCommand(int autoId, int commandId) {
                switch(autoId) {
                    case A2_ID:
                        if (A1_A2_Listener) {
                            switch(commandId) {
                                case A2Auto.A2_PATH_TRACED_COMMAND:
                                    A1_PathFinished = true;
                                    break;
                            }
                        }
                        break;
                }
            }
        }

        private class A2Auto extends ControlAutomaton {
            public static final int STATE_0 = 0;
            public static final int STATE_1 = 1;
            public static final int STATE_2 = 2;
            public static final int STATE_3 = 3;
            public static final int STATE_4 = 4;
            public static final int STATE_5 = 5;
            public static final int STATE_6 = 6;
            public static final int STATE_7 = 7;

            public static final int A2_PATH_TRACED_COMMAND = 1;
            public static final int A2_FIND_PATH_COMMAND = 2;

            public static final int CORRECT_TIME = 500;
            public static final int ROT_EX_TIME = 300;
            public static final int ROT_TIME = 12000;

            private boolean A2_A1_Listener = true;
            private boolean A2_TracePath = false;

            private boolean A2_A3_Listener = true;
            private boolean A2_PathLeft = false;
            private boolean A2_PathRight = false;

            public A2Auto(int id) {
                super(id);
                initialState = STATE_0;
            }

            public void init() {
                super.init();
                A2_A1_Listener = true;
                A2_TracePath = true;
                A2_A3_Listener = true;
                A2_PathLeft = false;
                A2_PathRight = false;
            }


            //----------------------------------------------------------------------------------------------------------
            private boolean x2_1() {
                return A2_TracePath;
            }

            private boolean x2_3() {
                return A2_PathLeft;
            }

            private boolean x2_4() {
                return A2_PathRight;
            }

            private boolean x2_7() {
                return (timer2active && (timer2 > CORRECT_TIME));
            }

            private boolean x2_8() {
                return (timer2active && (timer2 > ROT_EX_TIME));
            }

            private boolean x2_9() {
                return (timer2active && (timer2 > ROT_TIME));
            }

            //----------------------------------------------------------------------------------------------------------
            private void z2_3() {
                a1.addCommand(getId(), A2_PATH_TRACED_COMMAND);
            }

            private void z2_4_0() {
                A2_A1_Listener = false;
            }

            private void z2_4_1() {
                A2_A1_Listener = true;
            }

            private void z2_4_2() {
                A2_TracePath = false;
            }

            private void z2_5_0() {
                A2_A3_Listener = false;
            }

            private void z2_5_1() {
                A2_A3_Listener = true;
            }

            private void z2_5_2() {
                A2_PathLeft = false;
                A2_PathRight = false;
            }

            private void z2_6() {
                a3.addCommand(getId(), A2_FIND_PATH_COMMAND);
            }

            //----------------------------------------------------------------------------------------------------------
            public synchronized void addCommand(int autoId, int commandId) {
                switch(autoId) {
                    case A1_ID:
                        if (A2_A1_Listener) {
                            switch(commandId) {
                                case A1Auto.A1_TRACE_PATH_COMMAND:
                                    A2_TracePath = true;
                                    break;
                            }
                        }
                        break;

                    case A3_ID:
                        if (A2_A3_Listener) {
                            switch(commandId) {
                                case A3Auto.A3_PATH_LEFT_COMMAND:
                                    A2_PathLeft = true;
                                    break;

                                case A3Auto.A3_PATH_RIGHT_COMMAND:
                                    A2_PathRight = true;
                                    break;
                            }
                        }
                        break;
                }
            }

            //----------------------------------------------------------------------------------------------------------

            //no events and no data are used in this control system
            public synchronized void updateAutoState(int event, int data) {
                int oldState = getAutoState();

                switch(oldState) {
                    case STATE_0: {

                        //----------------------------------------------------------------------------------------------
                        boolean x2_1 = x2_1();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x2_1) {
                            setAutoState(STATE_1);
                        } else
                        if (!x2_1) {
                            setAutoState(STATE_0);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_1) {
                            z2_4_2();
                            z0_7();
                            z2_4_0();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_1: {
                        //----------------------------------------------------------------------------------------------
                        boolean x0_4_1 = x0_4_1();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x0_4_1) {
                            setAutoState(STATE_1);
                        } else
                        if (!x0_4_1) {
                            setAutoState(STATE_2);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_2: {
                        //----------------------------------------------------------------------------------------------
                        boolean x2_7 = x2_7();
                        boolean x0_4_0 = x0_4_0();
                        boolean x0_4_1 = x0_4_1();
                        boolean x0_4_2 = x0_4_2();
                        boolean x0_4_3 = x0_4_3();
                        boolean x0_5_0 = x0_5_0();
                        boolean x0_5_1 = x0_5_1();
                        boolean x0_5_2 = x0_5_2();
                        boolean x0_5_3 = x0_5_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_2();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x2_7 && x0_4_3) {
                            setAutoState(STATE_3);
                        } else

                        if (x2_7 && x0_4_1) {
                            setAutoState(STATE_1);
                        } else

                        if (x2_7 && x0_4_2) {
                            setAutoState(STATE_0);
                        } else

                        if (x2_7 && x0_4_0) {
                            if (x0_5_0) {
                                setAutoState(STATE_5);
                            } else

                            if (x0_5_3) {
                                setAutoState(STATE_3);
                            } else

                            if (x0_5_1 || x0_5_2) {
                                setAutoState(STATE_4);
                            }
                        } else

                        if (!x2_7) {
                            setAutoState(STATE_2);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_2();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_0) {
                            z2_4_1();
                            z2_3();
                            z0_9();
                        } else

                        if (getAutoState() == STATE_3) {
                            z0_9();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_3: {
                        if (stateChanged) {
                            stateChanged = false;
                        }

                        break;
                    }

                    case STATE_4: {
                        //----------------------------------------------------------------------------------------------
                        boolean x0_4_0 = x0_4_0();
                        boolean x0_4_1 = x0_4_1();
                        boolean x0_5_0 = x0_5_0();
                        boolean x0_5_1 = x0_5_1();
                        boolean x0_5_2 = x0_5_2();
                        boolean x0_5_3 = x0_5_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x0_4_1) {
                            setAutoState(STATE_1);
                        } else

                        if (!x0_4_0) {
                            setAutoState(STATE_2);
                        } else

                        if (x0_5_3) {
                            setAutoState(STATE_3);
                        } else

                        if (x0_5_0) {
                            setAutoState(STATE_5);
                        } else

                        if (x0_4_0 && (x0_5_1 || x0_5_2)) {
                            setAutoState(STATE_4);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_3) {
                            z0_9();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_5: {
                        //----------------------------------------------------------------------------------------------
                        boolean x2_3 = x2_3();
                        boolean x2_4 = x2_4();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z2_5_1();
                            z0_9();
                            z2_6();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        a3.updateAutoState(event, data);
                        //----------------------------------------------------------------------------------------------
                        if (a3.getAutoState() == A3Auto.STATE_5) {
                            setAutoState(STATE_3);
                        } else

                        if (x2_3) {
                            setAutoState(STATE_6);
                        } else

                        if (x2_4) {
                            setAutoState(STATE_6);
                        } else

                        if (!x2_3 && !x2_4) {
                            setAutoState(STATE_5);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z2_5_2();
                            z2_5_0();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_6) {
                            if (x2_3) {
                                z0_10();
                            } else

                            if (x2_4) {
                                z0_11();
                            }
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_6: {
                        //----------------------------------------------------------------------------------------------
                        boolean x2_9 = x2_9();
                        boolean x0_4_0 = x0_4_0();
                        boolean x0_4_1 = x0_4_1();
                        boolean x0_4_2 = x0_4_2();
                        boolean x0_4_3 = x0_4_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_2();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if ((x0_4_1 || x0_4_2) && !x2_9) {
                            setAutoState(STATE_7);
                        } else

                        if (x0_4_3 || x2_9) {
                            setAutoState(STATE_3);
                        } else

                        if (x0_4_0 && !x2_9) {
                            setAutoState(STATE_6);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_2();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_3) {
                            z0_9();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_7: {
                        //----------------------------------------------------------------------------------------------
                        boolean x2_8 = x2_8();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_2();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x2_8) {
                            setAutoState(STATE_1);
                        } else

                        if (!x2_8) {
                            setAutoState(STATE_7);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_2();
                            z0_9();
                            z0_7();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                }
            }

            public int getAutoState() {
                return state;
            }

            public void setAutoState(int state) {
                this.state = state;
            }
        }

        private class A3Auto extends ControlAutomaton {
            public static final int STATE_0 = 0;
            public static final int STATE_1 = 1;
            public static final int STATE_2 = 2;
            public static final int STATE_3 = 3;
            public static final int STATE_4 = 4;
            public static final int STATE_5 = 5;
            public static final int STATE_6 = 6;

            public static final int A3_PATH_LEFT_COMMAND = 1;
            public static final int A3_PATH_RIGHT_COMMAND = 2;
            public static final int A3_CENTER_ROD_COMMAND = 3;

            public static final int RR_TIME = 4000;
            public static final int ERR_POWER = 3;


            private boolean A3_A2_Listener = true;
            private boolean A3_Find_Path = false;

            private boolean A3_A5_Listener = true;
            private boolean A3_Rod_Centered = false;


            public A3Auto(int id) {
                super(id);
                initialState = STATE_0;
            }

            public void init() {
                super.init();
                A3_A2_Listener = true;
                A3_Find_Path = false;
                A3_A5_Listener = true;
                A3_Rod_Centered = false;
            }

            //----------------------------------------------------------------------------------------------------------
            private boolean x3_1() {
                return A3_Find_Path;
            }

            private boolean x3_3() {
                return (timer3active && (timer3 > RR_TIME));
            }

            private boolean x3_5() {
                return A3_Rod_Centered;
            }
            //----------------------------------------------------------------------------------------------------------
            private void z3_0() {
                a2.addCommand(getId(), A3_PATH_LEFT_COMMAND);
            }

            private void z3_1() {
                a2.addCommand(getId(), A3_PATH_RIGHT_COMMAND);
            }

            private void z3_2_0() {
                A3_A2_Listener = false;
            }

            private void z3_2_1() {
                A3_A2_Listener = true;
            }

            private void z3_2_2() {
                A3_Find_Path = false;
            }

            private void z3_4_0() {
                A3_A5_Listener = false;
            }

            private void z3_4_1() {
                A3_A5_Listener = true;
            }

            private void z3_4_2() {
                A3_Rod_Centered = false;
            }

            private void z3_5() {
                rod.turnLeft(ERR_POWER);
            }

            private void z3_6() {
                rod.turnRight(ERR_POWER);
            }

            private void z3_7() {
                rod.stop();
            }

            private void z3_8() {
                a5.addCommand(getId(), A3_CENTER_ROD_COMMAND);
            }

            public void addCommand(int autoId, int commandId) {
                switch (autoId) {
                    case A2_ID: {
                        if (A3_A2_Listener) {
                            switch(commandId) {
                                case A2Auto.A2_FIND_PATH_COMMAND:
                                    A3_Find_Path = true;
                                    break;
                            }
                        }

                        break;
                    }

                    case A5_ID: {
                        if (A3_A5_Listener) {
                            switch(commandId) {
                                case A5Auto.A5_ROD_CENTERED_COMMAND:
                                    A3_Rod_Centered = true;
                                    break;
                            }
                        }
                        break;
                    }
                }
            }

            //no events and no data are used in this control system
            public void updateAutoState(int event, int data) {
                int oldState = getAutoState();

                switch(oldState) {
                    case STATE_0: {
                        //----------------------------------------------------------------------------------------------
                        boolean x3_1 = x3_1();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z3_2_1();
                            z3_4_0();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x3_1) {
                            setAutoState(STATE_1);
                        } else

                        if (!x3_1) {
                            setAutoState(STATE_0);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z3_2_2();
                            z3_2_0();
                            z3_4_1();
                            z3_5();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_1: {
                        //----------------------------------------------------------------------------------------------
                        boolean x3_3 = x3_3();
                        boolean x0_5_0 = x0_5_0();
                        boolean x0_5_1 = x0_5_1();
                        boolean x0_5_2 = x0_5_2();
                        boolean x0_5_3 = x0_5_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_3();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x3_3) {
                            setAutoState(STATE_4);
                        } else

                        if (x0_5_1 | x0_5_2) {
                            setAutoState(STATE_2);
                        } else

                        if (x0_5_3) {
                            setAutoState(STATE_5);
                        } else

                        if (!x3_3 & x0_5_0) {
                            setAutoState(STATE_1);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_3();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_2) {
                            z3_8();
                        } else

                        if (getAutoState() == STATE_4) {
                            z3_8();
                        } else

                        if (getAutoState() == STATE_5) {
                            z3_7();
                        }
                        //----------------------------------------------------------------------------------------------

                        break;
                    }

                    case STATE_2:{
                        //----------------------------------------------------------------------------------------------
                        boolean x3_5 = x3_5();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        a5.updateAutoState(event, data);
                        //----------------------------------------------------------------------------------------------
                        if (a5.getAutoState() == A5Auto.STATE_3) {
                            setAutoState(STATE_5);
                        } else

                        if (x3_5) {
                            setAutoState(STATE_0);
                        } else

                        if (!x3_5) {
                            setAutoState(STATE_2);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_0) {
                            z3_4_2();
                            z3_0();
                        } else

                        if (getAutoState() == STATE_5) {
                            z3_7();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_3: {
                        //----------------------------------------------------------------------------------------------
                        boolean x3_3 = x3_3();
                        boolean x0_5_0 = x0_5_0();
                        boolean x0_5_1 = x0_5_1();
                        boolean x0_5_2 = x0_5_2();
                        boolean x0_5_3 = x0_5_3();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_3();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x0_5_3 || x3_3) {
                            setAutoState(STATE_5);
                        } else

                        if (x0_5_1 || x0_5_2) {
                            setAutoState(STATE_6);
                        } else

                        if (!x3_3 && x0_5_0) {
                            setAutoState(STATE_3);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_3();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_6) {
                            z3_8();
                        } else

                        if (getAutoState() == STATE_5) {
                            z3_7();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_4: {
                        //----------------------------------------------------------------------------------------------
                        boolean x3_5 = x3_5();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        a5.updateAutoState(event, data);
                        //----------------------------------------------------------------------------------------------
                        if (a5.getAutoState() == A5Auto.STATE_3) {
                            setAutoState(STATE_5);
                        } else

                        if (x3_5) {
                            setAutoState(STATE_3);
                        } else

                        if (!x3_5) {
                            setAutoState(STATE_4);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_3) {
                            z3_4_2();
                            z3_6();
                        } else

                        if (getAutoState() == STATE_5) {
                            z3_7();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_5: {
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_6: {
                        //----------------------------------------------------------------------------------------------
                        boolean x3_5 = x3_5();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        a5.updateAutoState(event, data);
                        //----------------------------------------------------------------------------------------------
                        if (a5.getAutoState() == A5Auto.STATE_3) {
                            setAutoState(STATE_5);
                        } else

                        if (x3_5) {
                            setAutoState(STATE_0);
                        } else

                        if (!x3_5) {
                            setAutoState(STATE_6);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_0) {
                            z3_4_2();
                            z3_1();
                        } else

                        if (getAutoState() == STATE_5) {
                            z3_7();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                }
            }

            public int getAutoState() {
                return state;
            }

            public void setAutoState(int state) {
                this.state = state;
            }
        }

        private class A5Auto extends ControlAutomaton {
            public static final int STATE_0 = 0;
            public static final int STATE_1 = 1;
            public static final int STATE_2 = 2;
            public static final int STATE_3 = 3;

            public static final int A5_ROD_CENTERED_COMMAND = 1;

            public static final int RB_TIME = 8000;

            private boolean A5_A3_Listener = true;
            private boolean A5_Center_Rod = false;

            public A5Auto(int id) {
                super(id);
                initialState = STATE_0;
            }

            public void init() {
                super.init();
                A5_A3_Listener = true;
                A5_Center_Rod = false;
            }

            private boolean x5_0() {
                return (touchSensor.getValue() == TouchSensor.TOUCHED_STATE);
            }

            private boolean x5_1() {
                return A5_Center_Rod;
            }

            private boolean x5_2() {
                return (timer4active && (timer4 > RB_TIME));
            }

            private void z5_1_0() {
                A5_A3_Listener = false;
            }

            private void z5_1_1() {
                A5_A3_Listener = true;
            }

            private void z5_1_2() {
                A5_Center_Rod = false;
            }

            private void z5_2() {
                rod.reverse();
            }

            private void z5_3() {
                rod.stop();
            }

            private void z5_4() {
                a3.addCommand(getId(), A5_ROD_CENTERED_COMMAND);
            }

            public void addCommand(int autoId, int command) {
                switch(autoId) {
                    case A3_ID: {
                        if (A5_A3_Listener) {
                            switch(command) {
                                case A3Auto.A3_CENTER_ROD_COMMAND:
                                    A5_Center_Rod = true;
                                    break;
                            }
                        }
                    }
                }
            }

            public synchronized void updateAutoState(int event, int data) {
                int oldState = getAutoState();

                switch(oldState) {
                    case STATE_0: {
                        //----------------------------------------------------------------------------------------------
                        boolean x5_1 = x5_1();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x5_1) {
                            setAutoState(STATE_1);
                        } else

                        if (!x5_1) {
                            setAutoState(STATE_0);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z5_1_2();
                            z5_1_0();
                            z5_2();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_1: {
                        //----------------------------------------------------------------------------------------------
                        boolean x5_2 = x5_2();
                        boolean x5_0 = x5_0();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_4();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x5_2) {
                            setAutoState(STATE_2);
                        } else

                        if (x5_0) {
                            setAutoState(STATE_0);
                        } else

                        if (!x5_2) {
                            setAutoState(STATE_1);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_4();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_2) {
                            z5_2();
                        } else

                        if (getAutoState() == STATE_0) {
                            z5_1_1();
                            z5_3();
                            z5_4();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_2: {
                        //----------------------------------------------------------------------------------------------
                        boolean x5_2 = x5_2();
                        boolean x5_0 = x5_0();
                        //----------------------------------------------------------------------------------------------
                        if (stateChanged) {
                            z0_5_4();
                            stateChanged = false;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (x5_2) {
                            setAutoState(STATE_3);
                        } else

                        if (x5_0) {
                            setAutoState(STATE_0);
                        } else

                        if (!x5_2) {
                            setAutoState(STATE_2);
                        }
                        //----------------------------------------------------------------------------------------------
                        if (oldState != getAutoState()) {
                            z0_6_4();
                            z5_3();
                            stateChanged = true;
                        }
                        //----------------------------------------------------------------------------------------------
                        if (getAutoState() == STATE_0) {
                            z5_1_1();
                            z5_4();
                        }
                        //----------------------------------------------------------------------------------------------
                        break;
                    }

                    case STATE_3: {
                        if (stateChanged) {
                            stateChanged = false;
                        }
                        break;
                    }
                }
            }

            public void setAutoState(int state) {
                this.state = state;
            }

            public int getAutoState() {
                return state;
            }
        }
    }

    public String toString() {
        String res = system.a1.toString() + "; ";
        res += system.a2.toString() + "; ";
        res += system.a3.toString() + "; ";
        res += system.a5.toString();
        return res;
    }
}