package robots.view;

import robots.transport.SixWheeledTransport;
import robots.sensors.LightSensor;
import robots.sensors.Sensor;
import robots.field.BattleField;
import robots.comm.InfraredField;
import robots.comm.InfraredReceiver;
import robots.comm.InfraredEmitter;
import robots.remote.RemoteControl;
import robots.dispencer.Dispencer;

import javax.swing.*;
import javax.swing.text.JTextComponent;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.ImageView;
import javax.swing.text.rtf.RTFEditorKit;
import java.awt.image.BufferedImage;
import java.awt.event.*;
import java.awt.geom.AffineTransform;
import java.awt.*;
import java.util.Iterator;
import java.io.*;
import java.net.URL;

import auto.Automaton;
import physics.Constants;
import physics.PhysicalObject;
import physics.Utilities;

/**
 * Boris Yartsev
 */
public class BattleFieldView extends JFrame implements Automaton {
    public static final int DEFAULT_FIELD_WIDTH = 500;
    public static final int DEFAULT_FIELD_HEIGHT = 500;

    public static final int DEFAULT_VIEWPORT_WIDTH = 550;
    public static final int DEFAULT_VIEWPORT_HEIGHT = 550;

    public static final int DEFAULT_CONTROL_WIDTH = 400;
    public static final int DEFAULT_CONTROL_HEIGHT = 550;

    private int battleWidth;
    private int battleHeight;

    private BattleField battleField;
    private BattleField.BattleFieldData data;

    private BattleFieldDrawPanel drawPanel;
    private ProcessControlPanel processControlPanel;
    private EditPanel editPanel;
    private BattleFieldEditor editor;

    private BufferedImage battleFieldDrawImage = null;

    //states of the BattleFieldView automaton
    private int state = SIMULATION_STOPPED_STATE;

    public static final int SIMULATION_STOPPED_STATE = 1;
    public static final int EDIT_STATE = 2;
    public static final int SIMULATION_STARTED_STATE = 3;
    public static final int SIMULATION_STOPPED_TRACK_STATE = 4;
    public static final int SIMULATION_STARTED_TRACK_STATE = 5;
    //end of the states of the BattleFieldView automaton


    public static final int E1_INITIALIZATION_EVENT = -1;
    public static final int E2_EDIT_BUTTON_PRESSED_EVENT = 1;
    public static final int E3_START_PAUSE_BUTTON_PRESSED_EVENT = 100;
    public static final int E4_STOP_BUTTON_PRESSED_EVENT = 102;
    public static final int E5_ZOOM_1_BUTTON_PRESSED_EVENT = 200;
    public static final int E6_ZOOM_2_BUTTON_PRESSED_EVENT = 201;
    public static final int E7_LEFT_BUTTON_PRESSED_EVENT = 302;
    public static final int E8_RIGHT_BUTTON_PRESSED_EVENT = 303;
    public static final int E9_DOWN_BUTTON_PRESSED_EVENT = 301;
    public static final int E10_UP_BUTTON_PRESSED_EVENT = 300;
    public static final int E11_CENTER_BUTTON_PRESSED_EVENT = 304;
    public static final int E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT = 305;
    public static final int E13_ONE_ITEM_BUTTON_PRESSED_EVENT = 400;
    public static final int E14_TWO_ITEM_BUTTON_PRESSED_EVENT = 401;
    public static final int E15_THREE_ITEM_BUTTON_PRESSED_EVENT = 402;
    public static final int E16_MOUSE_MOVED_EVENT = 5;
    public static final int E17_LEFT_MOUSE_DRAGGED_EVENT = 3;
    public static final int E18_RIGHT_MOUSE_DRAGGED_EVENT = 42;
    public static final int E19_LEFT_MOUSE_PRESSED_EVENT = 4;
    public static final int E20_LEFT_MOUSE_CLICKED_EVENT = 2;
    public static final int E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT = 7;
    public static final int E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT = 20;
    public static final int E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT = 31;
    public static final int E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT = 32;
    public static final int E25_DRAW_PATH_BUTTON_PRESSED_EVENT = 6;
    public static final int E26_DRAW_FOIL_BUTTON_PRESSED_EVENT = 16;
    public static final int E27_DRAW_ERROR_BUTTON_PRESSED_EVENT = 17;
    public static final int E28_CLEAR_BUTTON_PRESSED_EVENT = 18;
    public static final int E29_CLEAR_ALL_BUTTON_PRESSED_EVENT = 21;
    public static final int E30_OK_BUTTON_PRESSED_EVENT = 8;
    public static final int E31_CANCEL_BUTTON_PRESSED_EVENT = 9;
    public static final int E32_HELP_PATH_BUTTON_PRESSED_EVENT = 1010;
    public static final int E42_STOP_TRACK_EVENT = 1002;
    public static final int E43_STOP_SIMULATION_EVENT = 1003;

    private CursorImageFactory.Brush pathBrush = CursorImageFactory.createBrush(CursorImageFactory.PATH_BRUSH);
    private CursorImageFactory.Brush foilBrush = CursorImageFactory.createBrush(CursorImageFactory.FOIL_BRUSH);
    private CursorImageFactory.Brush errorBrush = CursorImageFactory.createBrush(CursorImageFactory.ERROR_BRUSH);
    private CursorImageFactory.Brush clearBrush = CursorImageFactory.createBrush(CursorImageFactory.FIELD_BRUSH);

    private Image pathImage = CursorImageFactory.createImage(CursorImageFactory.PATH_CURSOR_IMAGE);
    private Image foilImage = CursorImageFactory.createImage(CursorImageFactory.FOIL_CURSOR_IMAGE);
    private Image errorImage = CursorImageFactory.createImage(CursorImageFactory.ERROR_CURSOR_IMAGE);
    private Image clearImage = CursorImageFactory.createImage(CursorImageFactory.CLEAR_CURSOR_IMAGE);

    private Cursor pathCursor = Toolkit.getDefaultToolkit().createCustomCursor(pathImage, new Point(15, 15), "");
    private Cursor foilCursor = Toolkit.getDefaultToolkit().createCustomCursor(foilImage, new Point(15, 15), "");
    private Cursor errorCursor = Toolkit.getDefaultToolkit().createCustomCursor(errorImage, new Point(15, 15), "");
    private Cursor clearCursor = Toolkit.getDefaultToolkit().createCustomCursor(clearImage, new Point(15, 15), "");

    private int zoomRatio = 1;
    private int prevZoomRatio = 1;

    private int xshift = 0;
    private int yshift = 0;
    private int prevxshift;
    private int prevyshift;

    private int shiftCoeff = 20;

    private BattleFieldView(int battleWidth, int battleHeight, int viewPortWidth, int viewPortHeight, int controlWidth, int controlHeight) {

        this.battleWidth = battleWidth;
        this.battleHeight = battleHeight;

        battleField = createBattleField();
        data = battleField.createBattleFieldData();

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        getContentPane().setLayout(new BorderLayout());
        getContentPane().addComponentListener(new FieldComponentListener());

        drawPanel = new BattleFieldDrawPanel();
        drawPanel.setPreferredSize(new Dimension(viewPortWidth, viewPortHeight));
        getContentPane().add(drawPanel, BorderLayout.CENTER);

        processControlPanel = new ProcessControlPanel();
        processControlPanel.setPreferredSize(new Dimension(controlWidth, controlHeight));
        getContentPane().add(processControlPanel, BorderLayout.EAST);

        editPanel = new EditPanel();
        editPanel.setPreferredSize(new Dimension(controlWidth, controlHeight));

        editor = new BattleFieldEditor(pathBrush.getBrushImage(), clearBrush.getBrushImage(),
                foilBrush.getBrushImage(), errorBrush.getBrushImage(), clearBrush.getColor());

        pack();
        setResizable(true);
        setVisible(true);

        updateAutoState(E1_INITIALIZATION_EVENT, 0);
    }

    public class FieldComponentListener implements ComponentListener {
        public void componentResized(ComponentEvent e) {
            adjustPicture();
        }

        public void componentMoved(ComponentEvent e) {

        }

        public void componentShown(ComponentEvent e) {

        }

        public void componentHidden(ComponentEvent e) {

        }
    }

    public class EditPanel extends JPanel implements Automaton {
        private JToggleButton drawPathButton = new JToggleButton(
                new AbstractAction("Path", new ImageIcon("images\\icons\\pathicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E25_DRAW_PATH_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E25_DRAW_PATH_BUTTON_PRESSED_EVENT, 0);
                }
            });


        private JToggleButton drawFoilButton = new JToggleButton(
                new AbstractAction("Foil", new ImageIcon("images\\icons\\foilicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E26_DRAW_FOIL_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E26_DRAW_FOIL_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JToggleButton drawErrorButton = new JToggleButton(
                new AbstractAction("Error", new ImageIcon("images\\icons\\erroricon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E27_DRAW_ERROR_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E27_DRAW_ERROR_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JToggleButton clearButton = new JToggleButton(
                new AbstractAction("Clear", new ImageIcon("images\\icons\\clearicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E28_CLEAR_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E28_CLEAR_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton clearAllPathButton = new JButton(
                new AbstractAction("Clear All", new ImageIcon("images\\icons\\clearallicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E29_CLEAR_ALL_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E29_CLEAR_ALL_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JToggleButton setTransportButton = new JToggleButton(
                new AbstractAction("Place Transport", new ImageIcon("images\\icons\\placetransporticon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JToggleButton rotateTransportButton = new JToggleButton(
                new AbstractAction("Rotate Transport", new ImageIcon("images\\icons\\rotatetransporticon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JToggleButton setDispencerButton = new JToggleButton(
                new AbstractAction("Place Dispencer", new ImageIcon("images\\icons\\placedispencericon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JToggleButton rotateDispencerButton = new JToggleButton(
                new AbstractAction("Rotate Dispencer", new ImageIcon("images\\icons\\rotatedispencericon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton okButton = new JButton(new AbstractAction("OK", new ImageIcon("images\\icons\\okicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E30_OK_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E30_OK_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton cancelButton = new JButton(new AbstractAction("Cancel", new ImageIcon("images\\icons\\cancelicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E31_CANCEL_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E31_CANCEL_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton helpPathButton = new JButton(new AbstractAction("Help", new ImageIcon("images\\icons\\pathhelpicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E32_HELP_PATH_BUTTON_PRESSED_EVENT, 0);
                }
            }
        );

        private int state = IDLE_STATE;
        public static final int IDLE_STATE = 1;

        public EditPanel() {
            setLayout(new GridBagLayout());
            GridBagConstraints gc = new GridBagConstraints();
            gc.insets = new Insets(0, 10, 0, 10);
            gc.fill = GridBagConstraints.HORIZONTAL;

            gc.gridx = 0;
            gc.gridy = 1;
            gc.gridwidth = 2;
            gc.gridheight = 1;
            gc.weightx = 0;
            gc.weighty = 0;

            add(drawPathButton, gc);
            drawPathButton.setSelected(true);
            drawPathButton.setHorizontalAlignment(SwingConstants.LEFT);

            gc.gridy = 2;
            add(drawFoilButton, gc);
            drawFoilButton.setHorizontalAlignment(SwingConstants.LEFT);

            gc.gridy = 3;
            add(drawErrorButton, gc);
            drawErrorButton.setHorizontalAlignment(SwingConstants.LEFT);

            gc.gridy = 4;
            add(clearButton, gc);
            clearButton.setHorizontalAlignment(SwingConstants.LEFT);

            gc.insets = new Insets(10, 10, 10, 10);
            gc.gridy = 5;
            add(clearAllPathButton, gc);
            clearAllPathButton.setHorizontalAlignment(SwingConstants.LEFT);

            gc.gridy = 1;
            gc.gridx = 2;
            gc.insets = new Insets(0, 10, 0, 10);

            add(setTransportButton, gc);
            setTransportButton.setHorizontalAlignment(SwingConstants.LEFT);

            gc.gridy = 2;
            add(rotateTransportButton, gc);
            rotateTransportButton.setHorizontalAlignment(SwingConstants.LEFT);

            gc.gridy = 3;
            add(setDispencerButton, gc);
            setDispencerButton.setHorizontalAlignment(SwingConstants.LEFT);

            gc.gridy = 4;
            gc.insets = new Insets(0, 10, 0, 10);
            add(rotateDispencerButton, gc);
            rotateDispencerButton.setHorizontalAlignment(SwingConstants.LEFT);

            gc.gridy = 5;
            add(helpPathButton, gc);

            gc.gridx = 0;
            gc.gridy = 6;
            add(okButton, gc);

            gc.gridx = 2;
            add(cancelButton, gc);

        }

        public void updateAutoState(int event, int data) {
            switch(getAutoState()) {
                case IDLE_STATE: {
                    switch(event) {
                        case E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT:
                            z5_toggleTransportButton();
                            break;

                        case E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT:
                            z6_toggleDispenserButton();
                            break;

                        case E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT:
                            z8_toggleRotateTransportButton();
                            break;

                        case E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT:
                            z9_toggleRotateDispencerButton();
                            break;

                        case E25_DRAW_PATH_BUTTON_PRESSED_EVENT:
                            z1_togglePathButton();
                            break;

                        case E26_DRAW_FOIL_BUTTON_PRESSED_EVENT:
                            z2_toggleFoilButton();
                            break;

                        case E27_DRAW_ERROR_BUTTON_PRESSED_EVENT:
                            z3_toggleErrorButton();
                            break;

                        case E28_CLEAR_BUTTON_PRESSED_EVENT:
                            z4_toggleClearButton();
                            break;

                        case E30_OK_BUTTON_PRESSED_EVENT:
                            z1_togglePathButton();
                            break;

                        case E31_CANCEL_BUTTON_PRESSED_EVENT:
                            z1_togglePathButton();
                            break;

                        case E32_HELP_PATH_BUTTON_PRESSED_EVENT:
                            z10_showPathHelp();
                            break;
                    }
                    break;
                }
            }
        }

        public int getAutoState() {
            return state;
        }

        public void setAutoState(int state) {
            this.state = state;
        }

        private void z1_togglePathButton() {
            drawPathButton.setSelected(true);
            drawFoilButton.setSelected(false);
            drawErrorButton.setSelected(false);
            clearButton.setSelected(false);
            setTransportButton.setSelected(false);
            rotateTransportButton.setSelected(false);
            setDispencerButton.setSelected(false);
            rotateDispencerButton.setSelected(false);
        }

        private void z2_toggleFoilButton() {
            drawPathButton.setSelected(false);
            drawFoilButton.setSelected(true);
            drawErrorButton.setSelected(false);
            clearButton.setSelected(false);
            setTransportButton.setSelected(false);
            rotateTransportButton.setSelected(false);
            setDispencerButton.setSelected(false);
            rotateDispencerButton.setSelected(false);
        }

        private void z3_toggleErrorButton() {
            drawPathButton.setSelected(false);
            drawFoilButton.setSelected(false);
            drawErrorButton.setSelected(true);
            clearButton.setSelected(false);
            setTransportButton.setSelected(false);
            rotateTransportButton.setSelected(false);
            setDispencerButton.setSelected(false);
            rotateDispencerButton.setSelected(false);
        }

        private void z4_toggleClearButton() {
            drawPathButton.setSelected(false);
            drawFoilButton.setSelected(false);
            drawErrorButton.setSelected(false);
            clearButton.setSelected(true);
            setTransportButton.setSelected(false);
            rotateTransportButton.setSelected(false);
            setDispencerButton.setSelected(false);
            rotateDispencerButton.setSelected(false);
        }

        private void z5_toggleTransportButton() {
            drawPathButton.setSelected(false);
            drawFoilButton.setSelected(false);
            drawErrorButton.setSelected(false);
            clearButton.setSelected(false);
            setTransportButton.setSelected(true);
            rotateTransportButton.setSelected(false);
            setDispencerButton.setSelected(false);
            rotateDispencerButton.setSelected(false);
        }

        private void z6_toggleDispenserButton() {
            drawPathButton.setSelected(false);
            drawFoilButton.setSelected(false);
            drawErrorButton.setSelected(false);
            clearButton.setSelected(false);
            setTransportButton.setSelected(false);
            rotateTransportButton.setSelected(false);
            setDispencerButton.setSelected(true);
            rotateDispencerButton.setSelected(false);
        }

        private void z8_toggleRotateTransportButton() {
            drawPathButton.setSelected(false);
            drawFoilButton.setSelected(false);
            drawErrorButton.setSelected(false);
            clearButton.setSelected(false);
            setTransportButton.setSelected(false);
            rotateTransportButton.setSelected(true);
            setDispencerButton.setSelected(false);
            rotateDispencerButton.setSelected(false);
        }

        private void z9_toggleRotateDispencerButton() {
            drawPathButton.setSelected(false);
            drawFoilButton.setSelected(false);
            drawErrorButton.setSelected(false);
            clearButton.setSelected(false);
            setTransportButton.setSelected(false);
            rotateTransportButton.setSelected(false);
            setDispencerButton.setSelected(false);
            rotateDispencerButton.setSelected(true);
        }

        private void z10_showPathHelp() {
            final JDialog dialog = new JDialog(BattleFieldView.this, "Path help", true);

            JPanel panel = new JPanel();
            panel.setLayout(new BorderLayout());
            JEditorPane pane = new JEditorPane();
            InputStream stream = null;

            pane.setEditorKit(new HTMLEditorKit());
            //ImageView
            try {
                stream = new FileInputStream("help.html");
                pane.read(stream, null);
                //pane.getDocument().
            } catch (IOException e) {
                e.printStackTrace();
            }
            pane.setEditable(false);

            panel.add(pane, BorderLayout.CENTER);


            JPanel buttonPanel = new JPanel();
            buttonPanel.setLayout(new GridBagLayout());
            buttonPanel.setBackground(Color.WHITE);

            GridBagConstraints gc = new GridBagConstraints();
            gc.fill = GridBagConstraints.NONE;
            gc.insets = new Insets(10, 10, 10, 10);

            gc.gridx = 0;
            gc.gridy = 0;
            gc.gridwidth = 1;
            gc.gridheight = 1;
            gc.weightx = 0;
            gc.weighty = 0;

            buttonPanel.add(new JButton(new AbstractAction("OK", new ImageIcon("images\\icons\\okicon.gif")) {
                    public void actionPerformed(ActionEvent e) {
                        dialog.dispose();
                    }
                }), gc);

            panel.add(buttonPanel, BorderLayout.SOUTH);

            dialog.setContentPane(panel);
            dialog.pack();
            dialog.setResizable(false);
            dialog.setLocationRelativeTo(BattleFieldView.this);
            dialog.setVisible(true);
        }
    }

    public class ProcessControlPanel extends JPanel implements Automaton {
        private JButton startPauseButton = new JButton(new AbstractAction("Start", new ImageIcon("images\\icons\\runicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E3_START_PAUSE_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E3_START_PAUSE_BUTTON_PRESSED_EVENT, 0);
                }
            });

        ImageIcon startIcon = new ImageIcon("images\\icons\\runicon.gif");
        ImageIcon pauseIcon = new ImageIcon("images\\icons\\pauseicon.gif");

        private JButton stopButton = new JButton(new AbstractAction("Stop", new ImageIcon("images\\icons\\stopicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    BattleFieldView.this.updateAutoState(E4_STOP_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton editButton = new JButton(new AbstractAction("Edit", new ImageIcon("images\\icons\\editicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    BattleFieldView.this.updateAutoState(E2_EDIT_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton zoom1x = new JButton(new AbstractAction("Zoom", new ImageIcon("images\\icons\\zoom1icon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E5_ZOOM_1_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E5_ZOOM_1_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton zoom2x = new JButton(new AbstractAction("Zoom", new ImageIcon("images\\icons\\zoom2icon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    updateAutoState(E6_ZOOM_2_BUTTON_PRESSED_EVENT, 0);
                    BattleFieldView.this.updateAutoState(E6_ZOOM_2_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton oneItem = new JButton(
                new AbstractAction("One Item", new ImageIcon("images\\icons\\oneitem.gif")){
            public void actionPerformed(ActionEvent e) {
                updateAutoState(E13_ONE_ITEM_BUTTON_PRESSED_EVENT, 0);
                BattleFieldView.this.updateAutoState(E13_ONE_ITEM_BUTTON_PRESSED_EVENT, 0);
            }
        });

        private JButton twoItem = new JButton(
                new AbstractAction("Two Item", new ImageIcon("images\\icons\\twoitem.gif")){
            public void actionPerformed(ActionEvent e) {
                updateAutoState(E14_TWO_ITEM_BUTTON_PRESSED_EVENT, 0);
                BattleFieldView.this.updateAutoState(E14_TWO_ITEM_BUTTON_PRESSED_EVENT, 0);
            }
        });

        private JButton threeItem = new JButton(
                new AbstractAction("Three Item", new ImageIcon("images\\icons\\threeitem.gif")){
            public void actionPerformed(ActionEvent e) {
                updateAutoState(E15_THREE_ITEM_BUTTON_PRESSED_EVENT, 0);
                BattleFieldView.this.updateAutoState(E15_THREE_ITEM_BUTTON_PRESSED_EVENT, 0);
            }
        });

        private JButton up = new JButton(new AbstractAction("Up", new ImageIcon("images\\icons\\upicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    BattleFieldView.this.updateAutoState(E10_UP_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton down = new JButton(new AbstractAction("Down", new ImageIcon("images\\icons\\downicon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    BattleFieldView.this.updateAutoState(E9_DOWN_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton left = new JButton(new AbstractAction("Left", new ImageIcon("images\\icons\\lefticon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    BattleFieldView.this.updateAutoState(E7_LEFT_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton right = new JButton(new AbstractAction("Right", new ImageIcon("images\\icons\\righticon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    BattleFieldView.this.updateAutoState(E8_RIGHT_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JButton center = new JButton(new AbstractAction("Center", new ImageIcon("images\\icons\\centericon.gif")) {
                public void actionPerformed(ActionEvent e) {
                    BattleFieldView.this.updateAutoState(E11_CENTER_BUTTON_PRESSED_EVENT, 0);
                }
            });

        private JToggleButton trackTransport = new JToggleButton(new AbstractAction("Track Transport",
                new ImageIcon("images\\icons\\tracktransporticon.gif")) {
                    public void actionPerformed(ActionEvent e) {
                        updateAutoState(E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT, 0);
                        BattleFieldView.this.updateAutoState(E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT, 0);
                    }
                }
            );

        private RobotDataOutput tdo = new RobotDataOutput(battleField.getTransport());
        private RobotDataOutput ddo = new RobotDataOutput(battleField.getDispencer());

        public static final int TRACKING_START_STATE = 1;
        public static final int NOT_TRACKING_START_STATE = 2;
        public static final int TRACKING_PAUSE_STATE = 3;
        public static final int NOT_TRACKING_PAUSE_STATE = 4;

        private int state = NOT_TRACKING_START_STATE;

        public ProcessControlPanel() {
            setLayout(new GridBagLayout());
            GridBagConstraints gc = new GridBagConstraints();
            gc.insets = new Insets(5, 5, 5, 5);
            gc.fill = GridBagConstraints.HORIZONTAL;
            gc.anchor = GridBagConstraints.NORTH;

            gc.gridx = 0;
            gc.gridy = 0;
            gc.gridwidth = 2;
            gc.gridheight = 1;
            gc.weightx = 0;
            gc.weighty = 0;
            add(startPauseButton, gc);

            gc.gridy = 2;
            add(stopButton, gc);

            gc.gridy = 3;
            add(editButton, gc);

            gc.gridx = 2;
            gc.gridy = 0;
            add(zoom1x, gc);
            zoom1x.setEnabled(false);

            gc.gridy = 2;
            add(zoom2x, gc);

            gc.gridy = 5;
            add(oneItem, gc);
            oneItem.setEnabled(false);

            gc.gridy = 6;
            add(twoItem, gc);
            twoItem.setEnabled(false);

            gc.gridy = 7;
            add(threeItem, gc);
            threeItem.setEnabled(false);

            gc.gridx = 0;
            gc.gridy = 5;
            gc.gridwidth = 2;
            add(up, gc);

            gc.gridy = 6;
            gc.gridwidth = 1;
            add(left, gc);

            gc.gridx = 1;
            add(right, gc);

            gc.gridx = 0;
            gc.gridwidth = 2;
            gc.gridy = 7;
            add(down, gc);

            gc.gridy = 8;
            add(center, gc);

            gc.gridy = 9;
            add(trackTransport, gc);

            gc.gridx = 0; gc.gridy = 10; gc.gridwidth = 2; gc.gridheight = 2;
            add(tdo, gc);

            gc.gridx = 2;
            add(ddo, gc);
        }

        public synchronized void updateAutoState(int event, int data) {
            switch(getAutoState()) {
                case TRACKING_START_STATE: {
                    switch(event) {
                        case E3_START_PAUSE_BUTTON_PRESSED_EVENT:
                            z5_setPauseButton();
                            z8_enableItems();
                            setAutoState(TRACKING_PAUSE_STATE);
                            break;

                        case E5_ZOOM_1_BUTTON_PRESSED_EVENT:
                            z3_deactivateZoom1x();
                            break;

                        case E6_ZOOM_2_BUTTON_PRESSED_EVENT:
                            z4_deactivateZoom2x();
                            break;

                        case E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT:
                            z2_deactivateTrack();
                            setAutoState(NOT_TRACKING_START_STATE);
                            break;

                        case E42_STOP_TRACK_EVENT:
                            z2_deactivateTrack();
                            setAutoState(NOT_TRACKING_START_STATE);
                            break;
                    }

                    break;
                }

                case NOT_TRACKING_START_STATE: {
                    switch(event) {
                        case E3_START_PAUSE_BUTTON_PRESSED_EVENT:
                            z5_setPauseButton();
                            z8_enableItems();
                            setAutoState(NOT_TRACKING_PAUSE_STATE);
                            break;

                        case E5_ZOOM_1_BUTTON_PRESSED_EVENT:
                            z3_deactivateZoom1x();
                            break;

                        case E6_ZOOM_2_BUTTON_PRESSED_EVENT:
                            z4_deactivateZoom2x();
                            break;

                        case E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT:
                            z1_activateTrack();
                            setAutoState(TRACKING_START_STATE);
                            break;
                    }

                    break;
                }

                case TRACKING_PAUSE_STATE: {
                    switch(event) {
                        case E3_START_PAUSE_BUTTON_PRESSED_EVENT:
                            z6_setStartButton();
                            z7_disableItems();
                            setAutoState(TRACKING_START_STATE);
                            break;

                        case E5_ZOOM_1_BUTTON_PRESSED_EVENT:
                            z3_deactivateZoom1x();
                            break;

                        case E6_ZOOM_2_BUTTON_PRESSED_EVENT:
                            z4_deactivateZoom2x();
                            break;

                        case E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT:
                            z2_deactivateTrack();
                            setAutoState(NOT_TRACKING_PAUSE_STATE);
                            break;

                        case E42_STOP_TRACK_EVENT:
                            z2_deactivateTrack();
                            setAutoState(NOT_TRACKING_PAUSE_STATE);
                            break;

                        case E43_STOP_SIMULATION_EVENT:
                            z6_setStartButton();
                            z7_disableItems();
                            setAutoState(TRACKING_START_STATE);
                            break;
                    }
                    break;
                }

                case NOT_TRACKING_PAUSE_STATE: {
                    switch(event) {
                        case E3_START_PAUSE_BUTTON_PRESSED_EVENT:
                            z6_setStartButton();
                            z7_disableItems();
                            setAutoState(NOT_TRACKING_START_STATE);
                            break;

                        case E5_ZOOM_1_BUTTON_PRESSED_EVENT:
                            z3_deactivateZoom1x();
                            break;

                        case E6_ZOOM_2_BUTTON_PRESSED_EVENT:
                            z4_deactivateZoom2x();
                            break;

                        case E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT:
                            z1_activateTrack();
                            setAutoState(TRACKING_PAUSE_STATE);
                            break;

                        case E43_STOP_SIMULATION_EVENT:
                            z6_setStartButton();
                            z7_disableItems();
                            setAutoState(NOT_TRACKING_START_STATE);
                            break;
                    }
                    break;
                }
            }
        }

        public int getAutoState() {
            return state;
        }

        public void setAutoState(int state) {
            this.state = state;
        }

        private void z1_activateTrack() {
            up.setEnabled(false);
            down.setEnabled(false);
            left.setEnabled(false);
            right.setEnabled(false);
            center.setEnabled(false);

            trackTransport.setSelected(true);
        }

        private void z2_deactivateTrack() {
            up.setEnabled(true);
            down.setEnabled(true);
            left.setEnabled(true);
            right.setEnabled(true);
            center.setEnabled(true);

            trackTransport.setSelected(false);
        }

        private void z3_deactivateZoom1x() {
            zoom1x.setEnabled(false);
            zoom2x.setEnabled(true);
        }

        private void z4_deactivateZoom2x() {
            zoom1x.setEnabled(true);
            zoom2x.setEnabled(false);
        }

        private void z5_setPauseButton() {
            startPauseButton.setIcon(pauseIcon);
            startPauseButton.setText("Pause");
        }

        private void z6_setStartButton() {
            startPauseButton.setIcon(startIcon);
            startPauseButton.setText("Start");
        }

        private void z7_disableItems() {
            oneItem.setEnabled(false);
            twoItem.setEnabled(false);
            threeItem.setEnabled(false);
        }

        private void z8_enableItems() {
            oneItem.setEnabled(true);
            twoItem.setEnabled(true);
            threeItem.setEnabled(true);
        }
    }



    public static BattleFieldView createDefaultBattleFieldView() {
        return new BattleFieldView(DEFAULT_FIELD_WIDTH, DEFAULT_FIELD_HEIGHT,
                DEFAULT_VIEWPORT_WIDTH, DEFAULT_VIEWPORT_HEIGHT,
                DEFAULT_CONTROL_WIDTH, DEFAULT_CONTROL_HEIGHT);
    }

    private void adjustPicture() {
        int zoomRatio = getZoomRatio();
        int viewWidth = zoomRatio * drawPanel.getWidth();
        int w = battleWidth * zoomRatio;
        int viewHeight = zoomRatio * drawPanel.getHeight();
        int h = battleHeight * zoomRatio;

        if (viewWidth >= w) {
            setXShift(0);
        }

        if (viewHeight >= h) {
            setYShift(0);
        }
    }

    public int getZoomRatio() {
        return zoomRatio;
    }

    public void setZoomRatio(int zoomRatio) {
        this.zoomRatio = zoomRatio;
    }

    private int getXShift() {
        switch(getAutoState()) {
            case SIMULATION_STARTED_TRACK_STATE:
            case SIMULATION_STOPPED_TRACK_STATE:
                return (-transFieldToViewX(battleField.getTransport().getX()) + battleWidth / 2) * getZoomRatio();

            default:
                return xshift;
        }
    }

    private int getYShift() {
        switch(getAutoState()) {
            case SIMULATION_STARTED_TRACK_STATE:
            case SIMULATION_STOPPED_TRACK_STATE:
                return (-transFieldToViewY(battleField.getTransport().getY()) + battleHeight / 2) * getZoomRatio();
            default:
                return yshift;
        }
    }

    private void setXShift(int xshift) {
        this.xshift = xshift;
    }

    private void setYShift(int yshift) {
        this.yshift = yshift;
    }

    public int getAutoState() {
        return state;
    }

    public void setAutoState(int state) {
        this.state = state;
    }

    public synchronized void updateAutoState(int event, int data) {
        switch(getAutoState()) {
            case SIMULATION_STOPPED_STATE: {
                switch(event) {
                    case E1_INITIALIZATION_EVENT:
                        z4_createImage();
                        z8_loadImageToEditor();
                        z23_1_remoteButton1Prepared();
                        break;

                    case E2_EDIT_BUTTON_PRESSED_EVENT:
                        processControlPanel.updateAutoState(E43_STOP_SIMULATION_EVENT, 0);
                        z1_bringEditToFront();
                        z7_uploadParametersToEditor();
                        z9_setDrawPathCursor();
                        z32_1_storeZoomRatio();
                        z32_3_storeShiftValue();
                        setAutoState(EDIT_STATE);
                        break;

                    case E3_START_PAUSE_BUTTON_PRESSED_EVENT:
                        z30_1_start();
                        setAutoState(SIMULATION_STARTED_STATE);
                        break;

                    case E4_STOP_BUTTON_PRESSED_EVENT:
                        processControlPanel.updateAutoState(E43_STOP_SIMULATION_EVENT, 0);
                        z30_2_stop();
                        break;

                    case E5_ZOOM_1_BUTTON_PRESSED_EVENT:
                        z31_1_zoom1();
                        break;

                    case E6_ZOOM_2_BUTTON_PRESSED_EVENT:
                        z31_2_zoom2();
                        break;

                    case E7_LEFT_BUTTON_PRESSED_EVENT:
                        z33_1_moveLeftPicture();
                        break;

                    case E8_RIGHT_BUTTON_PRESSED_EVENT:
                        z33_2_moveRightPicture();
                        break;

                    case E9_DOWN_BUTTON_PRESSED_EVENT:
                        z33_4_moveDownPicture();
                        break;

                    case E10_UP_BUTTON_PRESSED_EVENT:
                        z33_3_moveUpPicture();
                        break;

                    case E11_CENTER_BUTTON_PRESSED_EVENT:
                        z33_5_centerPicture();
                        break;

                    case E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT:
                        z32_3_storeShiftValue();
                        setAutoState(SIMULATION_STOPPED_TRACK_STATE);
                        break;

                }
                break;
            }

            case SIMULATION_STARTED_STATE: {
                switch(event) {
                    case E2_EDIT_BUTTON_PRESSED_EVENT:
                        processControlPanel.updateAutoState(E43_STOP_SIMULATION_EVENT, 0);
                        z30_2_stop();
                        z1_bringEditToFront();
                        z7_uploadParametersToEditor();
                        z9_setDrawPathCursor();
                        z32_1_storeZoomRatio();
                        z32_3_storeShiftValue();
                        setAutoState(EDIT_STATE);
                        break;

                    case E3_START_PAUSE_BUTTON_PRESSED_EVENT:
                        z30_3_pause();
                        break;

                    case E4_STOP_BUTTON_PRESSED_EVENT:
                        processControlPanel.updateAutoState(E43_STOP_SIMULATION_EVENT, 0);
                        z30_2_stop();
                        setAutoState(SIMULATION_STOPPED_STATE);
                        break;

                    case E5_ZOOM_1_BUTTON_PRESSED_EVENT:
                        z31_1_zoom1();
                        break;

                    case E6_ZOOM_2_BUTTON_PRESSED_EVENT:
                        z31_2_zoom2();
                        break;

                    case E7_LEFT_BUTTON_PRESSED_EVENT:
                        z33_1_moveLeftPicture();
                        break;

                    case E8_RIGHT_BUTTON_PRESSED_EVENT:
                        z33_2_moveRightPicture();
                        break;

                    case E9_DOWN_BUTTON_PRESSED_EVENT:
                        z33_4_moveDownPicture();
                        break;

                    case E10_UP_BUTTON_PRESSED_EVENT:
                        z33_3_moveUpPicture();
                        break;

                    case E11_CENTER_BUTTON_PRESSED_EVENT:
                        z33_5_centerPicture();
                        break;

                    case E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT:
                        z32_3_storeShiftValue();
                        setAutoState(SIMULATION_STARTED_TRACK_STATE);
                        break;

                    case E13_ONE_ITEM_BUTTON_PRESSED_EVENT:
                        z23_1_remoteButton1Prepared();
                        break;

                    case E14_TWO_ITEM_BUTTON_PRESSED_EVENT:
                        z23_2_remoteButton2Prepared();
                        break;

                    case E15_THREE_ITEM_BUTTON_PRESSED_EVENT:
                        z23_3_remoteButton3Prepared();
                        break;

                    case E16_MOUSE_MOVED_EVENT:
                        z20_setRemotePosition((data >> 16) & 0xffff, data & 0xffff);
                        break;

                    case E17_LEFT_MOUSE_DRAGGED_EVENT:
                        z20_setRemotePosition((data >> 16) & 0xffff, data & 0xffff);
                        break;

                    case E18_RIGHT_MOUSE_DRAGGED_EVENT:
                        z21_setRemoteAngle((data >> 16) & 0xffff, data & 0xffff);
                        break;

                    case E19_LEFT_MOUSE_PRESSED_EVENT:
                        z22_remoteButtonPressed();
                        break;

                }

                break;
            }

            case SIMULATION_STARTED_TRACK_STATE: {
                switch(event) {
                    case E2_EDIT_BUTTON_PRESSED_EVENT:
                        processControlPanel.updateAutoState(E42_STOP_TRACK_EVENT, 0);
                        processControlPanel.updateAutoState(E43_STOP_SIMULATION_EVENT, 0);
                        z30_2_stop();
                        z1_bringEditToFront();
                        z7_uploadParametersToEditor();
                        z9_setDrawPathCursor();
                        z32_1_storeZoomRatio();
                        setAutoState(EDIT_STATE);
                        break;

                    case E3_START_PAUSE_BUTTON_PRESSED_EVENT:
                        z30_3_pause();
                        break;

                    case E4_STOP_BUTTON_PRESSED_EVENT:
                        processControlPanel.updateAutoState(E43_STOP_SIMULATION_EVENT, 0);
                        z30_2_stop();
                        setAutoState(SIMULATION_STOPPED_TRACK_STATE);
                        break;

                    case E5_ZOOM_1_BUTTON_PRESSED_EVENT:
                        z31_1_zoom1();
                        break;

                    case E6_ZOOM_2_BUTTON_PRESSED_EVENT:
                        z31_2_zoom2();
                        break;

                    case E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT:
                        z32_4_restoreShiftValue();
                        setAutoState(SIMULATION_STARTED_STATE);
                        break;

                    case E13_ONE_ITEM_BUTTON_PRESSED_EVENT:
                        z23_1_remoteButton1Prepared();
                        break;

                    case E14_TWO_ITEM_BUTTON_PRESSED_EVENT:
                        z23_2_remoteButton2Prepared();
                        break;

                    case E15_THREE_ITEM_BUTTON_PRESSED_EVENT:
                        z23_3_remoteButton3Prepared();
                        break;

                    case E16_MOUSE_MOVED_EVENT:
                        z20_setRemotePosition((data >> 16) & 0xffff, data & 0xffff);
                        break;

                    case E17_LEFT_MOUSE_DRAGGED_EVENT:
                        z20_setRemotePosition((data >> 16) & 0xffff, data & 0xffff);
                        break;

                    case E18_RIGHT_MOUSE_DRAGGED_EVENT:
                        z21_setRemoteAngle((data >> 16) & 0xffff, data & 0xffff);
                        break;

                    case E19_LEFT_MOUSE_PRESSED_EVENT:
                        z22_remoteButtonPressed();
                        break;

                }
                break;
            }

            case SIMULATION_STOPPED_TRACK_STATE: {
                switch(event) {
                    case E2_EDIT_BUTTON_PRESSED_EVENT:
                        processControlPanel.updateAutoState(E42_STOP_TRACK_EVENT, 0);
                        processControlPanel.updateAutoState(E43_STOP_SIMULATION_EVENT, 0);
                        z1_bringEditToFront();
                        z7_uploadParametersToEditor();
                        z9_setDrawPathCursor();
                        z32_1_storeZoomRatio();
                        setAutoState(EDIT_STATE);
                        break;

                    case E3_START_PAUSE_BUTTON_PRESSED_EVENT:
                        z30_1_start();
                        setAutoState(SIMULATION_STARTED_TRACK_STATE);
                        break;

                    case E4_STOP_BUTTON_PRESSED_EVENT:
                        processControlPanel.updateAutoState(E43_STOP_SIMULATION_EVENT, 0);
                        z30_2_stop();
                        break;

                    case E5_ZOOM_1_BUTTON_PRESSED_EVENT:
                        z31_1_zoom1();
                        break;

                    case E6_ZOOM_2_BUTTON_PRESSED_EVENT:
                        z31_2_zoom2();
                        break;

                    case E12_TRACK_TRANSPORT_BUTTON_PRESSED_EVENT:
                        z32_4_restoreShiftValue();
                        setAutoState(SIMULATION_STOPPED_STATE);
                        break;
                }
                break;
            }

            case EDIT_STATE: {
                switch(event) {
                    case E16_MOUSE_MOVED_EVENT:
                        editor.updateAutoState(E16_MOUSE_MOVED_EVENT, data);
                        break;

                    case E17_LEFT_MOUSE_DRAGGED_EVENT:
                        editor.updateAutoState(E17_LEFT_MOUSE_DRAGGED_EVENT, data);
                        break;

                    case E19_LEFT_MOUSE_PRESSED_EVENT:
                        editor.updateAutoState(E19_LEFT_MOUSE_PRESSED_EVENT, data);
                        break;

                    case E20_LEFT_MOUSE_CLICKED_EVENT:
                        editor.updateAutoState(E20_LEFT_MOUSE_CLICKED_EVENT, data);
                        break;

                    case E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT, 0);
                        z12_setEmptyCursor();
                        break;

                    case E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT, 0);
                        z12_setEmptyCursor();
                        break;

                    case E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT, 0);
                        z12_setEmptyCursor();
                        break;

                    case E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT, 0);
                        z12_setEmptyCursor();
                        break;

                    case E25_DRAW_PATH_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E25_DRAW_PATH_BUTTON_PRESSED_EVENT, 0);
                        z9_setDrawPathCursor();
                        break;

                    case E26_DRAW_FOIL_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E26_DRAW_FOIL_BUTTON_PRESSED_EVENT, 0);
                        z10_setDrawFoilCursor();
                        break;

                    case E27_DRAW_ERROR_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E27_DRAW_ERROR_BUTTON_PRESSED_EVENT, 0);
                        z11_setDrawErrorCursor();
                        break;

                    case E28_CLEAR_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E28_CLEAR_BUTTON_PRESSED_EVENT, 0);
                        z13_setClearCursor();
                        break;

                    case E29_CLEAR_ALL_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E29_CLEAR_ALL_BUTTON_PRESSED_EVENT, 0);
                        break;

                    case E30_OK_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E30_OK_BUTTON_PRESSED_EVENT, 0);
                        z12_setEmptyCursor();
                        z2_bringControlToFront();
                        z6_1_uploadParametersToBattleField();
                        z32_2_restoreZoomRatio();
                        z32_4_restoreShiftValue();
                        setAutoState(SIMULATION_STOPPED_STATE);
                        break;

                    case E31_CANCEL_BUTTON_PRESSED_EVENT:
                        editor.updateAutoState(E31_CANCEL_BUTTON_PRESSED_EVENT, 0);
                        z12_setEmptyCursor();
                        z2_bringControlToFront();
                        z6_2_restoreCoords();
                        z32_2_restoreZoomRatio();
                        z32_4_restoreShiftValue();
                        setAutoState(SIMULATION_STOPPED_STATE);
                        break;
                }

                break;
            }
        }
    }

    //------------------------------------------------------------------------------------------------------------------
    //------------------------------------------------------------------------------------------------------------------
    private void z1_bringEditToFront() {
        remove(processControlPanel);
        add(editPanel, BorderLayout.EAST);
        validate();
    }

    private void z2_bringControlToFront() {
        remove(editPanel);
        add(processControlPanel, BorderLayout.EAST);
        validate();
    }

    private void z4_createImage() {
        battleFieldDrawImage = new BufferedImage(battleWidth, battleHeight, BufferedImage.TYPE_INT_ARGB);
        Graphics g = battleFieldDrawImage.getGraphics();
        g.setColor(clearBrush.getColor());
        g.fillRect(0, 0, battleFieldDrawImage.getWidth(null), battleFieldDrawImage.getHeight(null));
    }

    private void z6_1_uploadParametersToBattleField() {
        int trX = editor.getRobotX(BattleFieldEditor.TRANSPORT_ROBOT);
        int trY = editor.getRobotY(BattleFieldEditor.TRANSPORT_ROBOT);
        double trAngle = editor.getRobotAngle(BattleFieldEditor.TRANSPORT_ROBOT);
        double newTrX = transViewToFieldX(trX);
        double newTrY = transViewToFieldY(trY);
        data.setTransportCoords(newTrX, newTrY, trAngle);

        int disX = editor.getRobotX(BattleFieldEditor.DISPENCER_ROBOT);
        int disY = editor.getRobotY(BattleFieldEditor.DISPENCER_ROBOT);
        double disAngle = editor.getRobotAngle(BattleFieldEditor.DISPENCER_ROBOT);
        double newDisX = transViewToFieldX(disX);
        double newDisY = transViewToFieldY(disY);
        data.setDispencerCoords(newDisX, newDisY, disAngle);

        int [] pixels = new int[4];
        for (int i = 0; i < battleWidth; i++) {
            for (int j = battleHeight - 1; j >= 0; j--) {
                battleFieldDrawImage.getRaster().getPixel(i, j, pixels);
                int type = BattleField.BattleFieldData.FIELD;

                Color pathColor = pathBrush.getColor();
                Color fieldColor = clearBrush.getColor();
                Color errorColor = errorBrush.getColor();
                Color foilColor = foilBrush.getColor();

                //path
                if ((pixels[0] == pathColor.getRed()) &&
                        (pixels[1] == pathColor.getGreen()) &&
                        (pixels[2] == pathColor.getBlue())) {
                    type = BattleField.BattleFieldData.PATH;
                } else
                //field
                if ((pixels[0] == fieldColor.getRed()) &&
                        (pixels[1] == fieldColor.getGreen()) &&
                        (pixels[2] == fieldColor.getBlue())) {
                    type = BattleField.BattleFieldData.FIELD;
                } else

                if (pixels[0] == foilColor.getRed() &&
                        pixels[1] == foilColor.getGreen() &&
                        pixels[2] == foilColor.getBlue()) {
                    type = BattleField.BattleFieldData.FOIL;
                } else

                if (pixels[0] == errorColor.getRed() &&
                        pixels[1] == errorColor.getGreen() &&
                        pixels[2] == errorColor.getBlue()) {
                    type = BattleField.BattleFieldData.ERROR;
                }

                data.setPointAt(i, battleHeight - 1 - j, type);
            }
        }
        battleField.loadNewData(data);
    }

    private void z6_2_restoreCoords() {
        battleField.loadNewData(data);
    }

    private void z7_uploadParametersToEditor() {
        editor.setRobotCoords(BattleFieldEditor.TRANSPORT_ROBOT,
                transFieldToViewX(data.getTransportX()), transFieldToViewY(data.getTransportY()), data.getTransportAngle());
        editor.setRobotCoords(BattleFieldEditor.DISPENCER_ROBOT,
                transFieldToViewX(data.getDispencerX()), transFieldToViewY(data.getDispencerY()), data.getDispencerAngle());
    }

    private void z8_loadImageToEditor() {
        editor.loadNewImage(battleFieldDrawImage);
    }

    private void z9_setDrawPathCursor() {
        drawPanel.setCursor(pathCursor);
    }

    private void z10_setDrawFoilCursor() {
        drawPanel.setCursor(foilCursor);
    }

    private void z11_setDrawErrorCursor() {
        drawPanel.setCursor(errorCursor);
    }

    private void z12_setEmptyCursor() {
        drawPanel.setCursor(null);
    }

    private void z13_setClearCursor() {
        drawPanel.setCursor(clearCursor);
    }

    private void z20_setRemotePosition(int x, int y) {
        battleField.getRemoteControl().setX(transViewToFieldX(x));
        battleField.getRemoteControl().setY(transViewToFieldY(y));
    }

    private void z21_setRemoteAngle(int x, int y) {
        RemoteControl rc = battleField.getRemoteControl();

        double r1x = rc.getX();
        double r1y = rc.getY();
        double r2x = transViewToFieldX(x);
        double r2y = transViewToFieldY(y);

        double angle = Utilities.convertToAngle(r2x - r1x, r2y - r1y);
        rc.setAngle(angle);
    }

    private void z22_remoteButtonPressed() {
        battleField.getRemoteControl().buttonPressed();
    }

    private void z23_1_remoteButton1Prepared() {
        battleField.getRemoteControl().prepareButton(RemoteControl.REMOTE_BUTTON1);
    }

    private void z23_2_remoteButton2Prepared() {
        battleField.getRemoteControl().prepareButton(RemoteControl.REMOTE_BUTTON2);
    }

    private void z23_3_remoteButton3Prepared() {
        battleField.getRemoteControl().prepareButton(RemoteControl.REMOTE_BUTTON3);
    }

    private void z30_1_start() {
        battleField.start();
    }

    private void z30_2_stop() {
        battleField.stop();
    }

    private void z30_3_pause() {
        battleField.pause();
    }

    private void z31_1_zoom1() {
        setZoomRatio(1);
        adjustPicture();
    }

    private void z31_2_zoom2() {
        setZoomRatio(2);
        adjustPicture();
    }

    private void z32_1_storeZoomRatio() {
        prevZoomRatio = getZoomRatio();
        setZoomRatio(1);
    }

    private void z32_2_restoreZoomRatio() {
        setZoomRatio(prevZoomRatio);
    }

    private void z32_3_storeShiftValue() {
        prevxshift = xshift;
        setXShift(0);
        prevyshift = yshift;
        setYShift(0);
    }

    private void z32_4_restoreShiftValue() {
        setXShift(prevxshift);
        setYShift(prevyshift);
    }


    private void z33_1_moveLeftPicture() {
        int viewWidth = drawPanel.getWidth();
        int w = battleWidth * getZoomRatio();

        if ((viewWidth - w) / 2 + getXShift() < 0) {
            setXShift(getXShift() + getZoomRatio() * shiftCoeff);
        }
    }

    private void z33_2_moveRightPicture() {
        int viewWidth = drawPanel.getWidth();
        int w = battleWidth * getZoomRatio();

        if ((-viewWidth + w) / 2 + getXShift() > 0) {
            setXShift(getXShift() - getZoomRatio() * shiftCoeff);
        }
    }

    private void z33_3_moveUpPicture() {
        int viewHeight = drawPanel.getHeight();
        int h = battleHeight * getZoomRatio();

        if ((viewHeight - h) / 2 + getYShift() < 0) {
            setYShift(getYShift() + getZoomRatio() * shiftCoeff);
        }
    }

    private void z33_4_moveDownPicture() {
        int viewHeight = drawPanel.getHeight();
        int h = battleHeight * getZoomRatio();

        if ((-viewHeight + h) / 2 + getYShift() > 0) {
            setYShift(getYShift() - getZoomRatio() * shiftCoeff);
        }
    }

    private void z33_5_centerPicture() {
        setXShift(0);
        setYShift(0);
    }

    //------------------------------------------------------------------------------------------------------------------
    //------------------------------------------------------------------------------------------------------------------

    private BattleField createBattleField() {
        return new BattleField(battleWidth, battleHeight);
    }

    //These functions are used for converting the coordinates back and forth from the viewPort to the battleField
    //and from the battleField to the viewPort
    private int transFieldToViewX(double x) {
        return (int) x;
    }

    private int transFieldToViewY(double y) {
        return (int) (battleWidth - y);
    }

    private double transViewToFieldX(int x) {
        return x;
    }

    private double transViewToFieldY(int y) {
        return battleWidth - y;
    }

    private int transScreenToViewX(int x) {
        int zoomRatio = getZoomRatio();
        return (x - getXShift() - (drawPanel.getWidth() - battleWidth * zoomRatio) / 2) / zoomRatio;
    }

    private int transScreenToViewY(int y) {
        int zoomRatio = getZoomRatio();
        return (y - getYShift() - (drawPanel.getHeight() - battleHeight * zoomRatio) / 2) / zoomRatio;
    }

    private class BattleFieldDrawPanel extends JPanel implements MouseListener, MouseMotionListener {

        private AffineTransform trans = new AffineTransform();

        private Composite alphaComposite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);

        private BufferedImage buffer = null;
        private BufferedImage buffer2x = null;
        private int array[] = new int[3];

        public BattleFieldDrawPanel() {
            addMouseListener(this);
            addMouseMotionListener(this);
        }

        private void drawImageAtPosition(Graphics2D g, Image img, double x, double y, double angle) {
            if (img != null) {
                trans.setToIdentity();
                trans.translate(x - img.getWidth(null) / 2, y - img.getHeight(null) / 2);
                trans.rotate(-angle, img.getWidth(null) / 2, img.getHeight(null) / 2);
                g.drawImage(img, trans, null);
            }
        }

        private SixWheeledTransport getTransport() {
            switch(BattleFieldView.this.getAutoState()) {
                case SIMULATION_STOPPED_STATE:
                case SIMULATION_STOPPED_TRACK_STATE:
                case SIMULATION_STARTED_STATE:
                case SIMULATION_STARTED_TRACK_STATE:
                    return battleField.getTransport();

                case EDIT_STATE:
                    SixWheeledTransport transport = battleField.getTransport();
                    transport.setX(transViewToFieldX(editor.getRobotX(BattleFieldEditor.TRANSPORT_ROBOT)));
                    transport.setY(transViewToFieldY(editor.getRobotY(BattleFieldEditor.TRANSPORT_ROBOT)));
                    transport.setAngle(editor.getRobotAngle(BattleFieldEditor.TRANSPORT_ROBOT));
                    return transport;
            }
            return null;
        }

        private Dispencer getDispencer() {
            switch(BattleFieldView.this.getAutoState()) {
                case SIMULATION_STOPPED_STATE:
                case SIMULATION_STOPPED_TRACK_STATE:
                case SIMULATION_STARTED_STATE:
                case SIMULATION_STARTED_TRACK_STATE:
                    return battleField.getDispencer();

                case EDIT_STATE:
                    Dispencer dispencer = battleField.getDispencer();
                    dispencer.setX(transViewToFieldX(editor.getRobotX(BattleFieldEditor.DISPENCER_ROBOT)));
                    dispencer.setY(transViewToFieldY(editor.getRobotY(BattleFieldEditor.DISPENCER_ROBOT)));
                    dispencer.setAngle(editor.getRobotAngle(BattleFieldEditor.DISPENCER_ROBOT));
                    return dispencer;
            }
            return null;
        }

        public void paintComponent(Graphics g) {
            int w = battleWidth;
            int h = battleHeight;

            if (buffer == null) {
//                buffer = createImage(w, h);
                buffer = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
                buffer2x = new BufferedImage(2 * w, 2 * h, BufferedImage.TYPE_INT_RGB);
            }


            Graphics2D ig = (Graphics2D) buffer.getGraphics();
            ig.drawImage(battleFieldDrawImage, 0, 0, null);

            //Transport drawing
            SixWheeledTransport transport = getTransport();

            if (transport.isShown()) {
                drawImageAtPosition(ig, transport.getObjectImage(),
                        transFieldToViewX(transport.getX()),
                        transFieldToViewY(transport.getY()),
                        transport.getAngle());
            }

            java.util.List sensorList = transport.getSensors();

            for (Iterator iter = sensorList.iterator(); iter.hasNext(); ) {
                Sensor sensor = (Sensor) iter.next();

                if (sensor instanceof LightSensor) {
                    LightSensor ls = (LightSensor) sensor;
                    if (ls.isShown()) {
                        drawImageAtPosition(ig, ls.getObjectImage(), transFieldToViewX(ls.getX()),
                                transFieldToViewY(ls.getY()), ls.getAngle());
                    }
                }
            }
            //end transport drawing

            //dispencer drawing
            Dispencer dispencer = getDispencer();

            if (dispencer.isShown()) {
                drawImageAtPosition(ig, dispencer.getObjectImage(),
                        transFieldToViewX(dispencer.getX()),
                        transFieldToViewY(dispencer.getY()),
                        dispencer.getAngle());
            }
            //end dispencer drawing

            //remote drawing
            RemoteControl remote = battleField.getRemoteControl();

            switch(getAutoState()) {
                case SIMULATION_STARTED_STATE:
                case SIMULATION_STARTED_TRACK_STATE:

                    if (remote.isShown()) {
                        drawImageAtPosition(ig, remote.getObjectImage(),
                                transFieldToViewX(remote.getX()),
                                transFieldToViewY(remote.getY()),
                                remote.getAngle());
                    }

                    break;
            }
            //end remote drawing

            InfraredField irfield = battleField.getInfraredField();

            //drawing the receivers on the field
            java.util.List receivers = irfield.getReceivers();

            for (Iterator iter = receivers.iterator(); iter.hasNext();) {
                InfraredReceiver rec = (InfraredReceiver) iter.next();

                if (rec.isShown()) {
                    Image recImg = rec.getObjectImage();

                    if (recImg != null) {
                        int recX = transFieldToViewX(rec.getX());
                        int recY = transFieldToViewY(rec.getY());

                        double recAngle = rec.getAngle();

                        trans.setToIdentity();
                        trans.translate(recX - recImg.getWidth(null) / 2, recY - recImg.getHeight(null) / 2);
                        trans.rotate(-recAngle, recImg.getWidth(null) / 2, recImg.getHeight(null) / 2);

                        ig.drawImage(recImg, trans, null);
                    }
                }
            }
            //end drawing the receivers on the field

            //drawing the emitters on the field
            java.util.List emitters = irfield.getEmitters();

            for (Iterator iter = emitters.iterator(); iter.hasNext();) {
                InfraredEmitter em = (InfraredEmitter) iter.next();

                if (em.isShown()) {
                    Image emImg = em.getObjectImage();

                    if (emImg != null) {
                        int emX = transFieldToViewX(em.getX());
                        int emY = transFieldToViewY(em.getY());

                        double emAngle = em.getAngle();

                        trans.setToIdentity();
                        trans.translate(emX - emImg.getWidth(null) / 2, emY - emImg.getHeight(null) / 2);
                        trans.rotate(-emAngle, emImg.getWidth(null) / 2, emImg.getHeight(null) / 2);

                        ig.drawImage(emImg, trans, null);
                    }
                }
            }
            //end drawing the emitters on the field

            //Drawing the infrared signals on the field
            Composite oldComposite = ig.getComposite();
            ig.setComposite(alphaComposite);

            switch(getAutoState()) {
                case SIMULATION_STARTED_STATE:
                case SIMULATION_STARTED_TRACK_STATE:

                    for (Iterator iter = emitters.iterator(); iter.hasNext();) {
                        InfraredEmitter em = (InfraredEmitter) iter.next();

                        if (em.isSending()) {
                            int startAngle = Utilities.convertRad2Angle(em.getAngle() - em.getInfraredAlpha() / 2);
                            int arcAngle = Utilities.convertRad2Angle(em.getInfraredAlpha());

                            int emX = transFieldToViewX(em.getX());
                            int emY = transFieldToViewY(em.getY());
                            int rad = (int) Math.round(em.getInfraredRadius());

                            ig.setColor(em.getEmitterColor());
                            ig.fillArc(emX - rad, emY - rad, 2 * rad, 2 * rad, startAngle, arcAngle);
                        }
                    }
            }

            ig.setComposite(oldComposite);
            //end drawing the infrared signals in the field


            //drawing the offscreen buffer to the output
            g.setColor(Color.BLACK);
            int viewportWidth = drawPanel.getWidth();
            int viewportHeight = drawPanel.getHeight();
            g.fillRect(0, 0, viewportWidth, viewportHeight);

            //offscreen image rendering
            int zoomRatio = getZoomRatio();

            int dx = getXShift();
            int dy = getYShift();

            switch(zoomRatio) {
                case 1:
                    g.drawImage(buffer,
                            dx + (viewportWidth - w) / 2, dy + (viewportHeight - h) / 2, null);
                    break;

                case 2:

                    int nw = w * zoomRatio;
                    int nh = h * zoomRatio;

                    int wStart = ((nw - viewportWidth) / 2 - getXShift()) / 2;
                    int wEnd = wStart + viewportWidth / 2;
                    int hStart = ((nh - viewportHeight) / 2 - getYShift()) / 2;
                    int hEnd = hStart + viewportHeight / 2;

                    if (wStart < 0) {
                        wStart = 0;
                    }

                    if (wStart > w) {
                        wStart = w;
                    }

                    if (wEnd > w) {
                        wEnd = w;
                    }

                    if (wEnd < 0) {
                        wEnd = 0;
                    }

                    if (hStart < 0) {
                        hStart = 0;
                    }

                    if (hStart > h) {
                        hStart = h;
                    }

                    if (hEnd > h) {
                        hEnd = h;
                    }

                    if (hEnd < 0) {
                        hEnd = 0;
                    }

                    for (int j = wStart; j < wEnd; j++) {
                        for (int k = hStart; k < hEnd; k++) {
                            buffer.getRaster().getPixel(j, k, array);
                            buffer2x.getRaster().setPixel(2 * j, 2 * k, array);
                            buffer2x.getRaster().setPixel(2 * j + 1, 2 * k, array);
                            buffer2x.getRaster().setPixel(2 * j, 2 * k + 1, array);
                            buffer2x.getRaster().setPixel(2 * j + 1, 2 * k + 1, array);
                        }
                    }

                    g.drawImage(buffer2x, dx + (viewportWidth - nw) / 2, dy + (viewportHeight - nh) / 2, null);
                    break;
            }
        }

        /*
        public void paintComponent(Graphics g) {
            int zoomRatio = getZoomRatio();

            int w = battleWidth * zoomRatio;
            int h = battleHeight * zoomRatio;


            Image fieldImage = null;

            if (getAutoState() == EDIT_STATE) {
                fieldImage = battleFieldDrawImage;
            } else {
                fieldImage = fieldZoomer.getBufferImageOfFactor(zoomRatio);
            }

            Graphics2D ig = (Graphics2D) fieldImage.getGraphics();

            //Transport drawing
            SixWheeledTransport transport = getTransport();

            if (transport.isShown()) {
                transport.setZoomRatio(zoomRatio);

                drawImageAtPosition(ig, transport.getObjectImage(),
                        transFieldToViewX(transport.getX()) * zoomRatio,
                        transFieldToViewY(transport.getY()) * zoomRatio,
                        transport.getAngle());
            }

            java.util.List sensorList = transport.getSensors();
//            int i = 0;

            for (Iterator iter = sensorList.iterator(); iter.hasNext(); ) {
                Sensor sensor = (Sensor) iter.next();

                if (sensor instanceof LightSensor) {
                    LightSensor ls = (LightSensor) sensor;
                    if (ls.isShown()) {
                        ls.setZoomRatio(zoomRatio);
                        drawImageAtPosition(ig, ls.getObjectImage(), transFieldToViewX(ls.getX()) * zoomRatio,
                                transFieldToViewY(ls.getY()) * zoomRatio, ls.getAngle());
                    }
                }

                ig.setColor(Color.BLACK);
//                ig.drawString(sensor.toString(), 10, 10 + i * 20);
//                i++;
            }
            //end transport drawing

            //dispencer drawing
            Dispencer dispencer = getDispencer();

            if (dispencer.isShown()) {
                dispencer.setZoomRatio(zoomRatio);

                drawImageAtPosition(ig, dispencer.getObjectImage(),
                        transFieldToViewX(dispencer.getX()) * zoomRatio,
                        transFieldToViewY(dispencer.getY()) * zoomRatio,
                        dispencer.getAngle());
            }
            //end dispencer drawing

            //remote drawing
            if (battleField.isSimulation()) {
                RemoteControl remote = battleField.getRemoteControl();
                if (remote.isShown()) {
                    remote.setZoomRatio(zoomRatio);
                    drawImageAtPosition(ig, remote.getObjectImage(),
                            transFieldToViewX(remote.getX()) * zoomRatio,
                            transFieldToViewY(remote.getY()) * zoomRatio, remote.getAngle());
                }
            }
            //end remote drawing

            InfraredField irfield = battleField.getInfraredField();

            //drawing the receivers on the field
            java.util.List receivers = irfield.getReceivers();

            for (Iterator iter = receivers.iterator(); iter.hasNext();) {
                InfraredReceiver rec = (InfraredReceiver) iter.next();

                if (rec.isShown()) {
                    rec.setZoomRatio(zoomRatio);
                    Image recImg = rec.getObjectImage();

                    if (recImg != null) {
                        int recX = transFieldToViewX(rec.getX()) * zoomRatio;
                        int recY = transFieldToViewY(rec.getY()) * zoomRatio;

                        double recAngle = rec.getAngle();

                        trans.setToIdentity();
                        trans.translate(recX - recImg.getWidth(null) / 2, recY - recImg.getHeight(null) / 2);
                        trans.rotate(-recAngle, recImg.getWidth(null) / 2, recImg.getHeight(null) / 2);

                        ig.drawImage(recImg, trans, null);
                    }
                }
            }
            //end drawing the receivers on the field

            //drawing the emitters on the field
            java.util.List emitters = irfield.getEmitters();

            for (Iterator iter = emitters.iterator(); iter.hasNext();) {
                InfraredEmitter em = (InfraredEmitter) iter.next();

                if (em.isShown()) {
                    em.setZoomRatio(zoomRatio);
                    Image emImg = em.getObjectImage();

                    if (emImg != null) {
                        int emX = transFieldToViewX(em.getX()) * zoomRatio;
                        int emY = transFieldToViewY(em.getY()) * zoomRatio;

                        double emAngle = em.getAngle();

                        trans.setToIdentity();
                        trans.translate(emX - emImg.getWidth(null) / 2, emY - emImg.getHeight(null) / 2);
                        trans.rotate(-emAngle, emImg.getWidth(null) / 2, emImg.getHeight(null) / 2);

                        ig.drawImage(emImg, trans, null);
                    }
                }
            }
            //end drawing the emitters on the field

            //Drawing the infrared signals on the field
            Composite oldComposite = ig.getComposite();
            ig.setComposite(alphaComposite);

            for (Iterator iter = emitters.iterator(); iter.hasNext();) {
                InfraredEmitter em = (InfraredEmitter) iter.next();

                if (em.isSending()) {
                    int startAngle = Utilities.convertRad2Angle(em.getAngle() - em.getInfraredAlpha() / 2);
                    int arcAngle = Utilities.convertRad2Angle(em.getInfraredAlpha());

                    int emX = transFieldToViewX(em.getX()) * zoomRatio;
                    int emY = transFieldToViewY(em.getY()) * zoomRatio;
                    int rad = (int) Math.round(em.getInfraredRadius()) * zoomRatio;

                    ig.setColor(em.getEmitterColor());
                    ig.fillArc(emX - rad, emY - rad, 2 * rad, 2 * rad, startAngle, arcAngle);
                }
            }

            ig.setComposite(oldComposite);
            //end drawing the infrared signals in the field

            //Data output
//            ig.setColor(Color.BLACK);
//            ig.drawString(transport.toString(), 10, 90);
//            ig.drawString(dispencer.toString(), 10, 110);

            //drawing the offscreen buffer to the output
            g.setColor(Color.BLACK);
            int viewportWidth = drawPanel.getWidth();
            int viewportHeight = drawPanel.getHeight();
            g.fillRect(0, 0, viewportWidth, viewportHeight);

            fieldImage.setAccelerationPriority(1);
            //offscreen image rendering
            g.drawImage(fieldImage, xshift + (viewportWidth - w) / 2, yshift + (viewportHeight - h) / 2, null);
        }
        */

        //this is needed for placing the transport, dispencer and foil pieces on the field
        public void mouseClicked(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON1) {
                int xCoord = transScreenToViewX(e.getX());
                int yCoord = transScreenToViewY(e.getY());
                BattleFieldView.this.updateAutoState(E20_LEFT_MOUSE_CLICKED_EVENT, (xCoord << 16) + yCoord);
            }
        }

        //these two events needed for drawing the lines,
        public void mousePressed(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON1) {
                int xCoord = transScreenToViewX(e.getX());
                int yCoord = transScreenToViewY(e.getY());
                BattleFieldView.this.updateAutoState(E19_LEFT_MOUSE_PRESSED_EVENT, (xCoord << 16) + yCoord);
            }
        }

        public void mouseDragged(MouseEvent e) {
            int xCoord = transScreenToViewX(e.getX());
            int yCoord = transScreenToViewY(e.getY());

            if ((e.getModifiersEx() & MouseEvent.BUTTON1_DOWN_MASK) > 0) {
                BattleFieldView.this.updateAutoState(E17_LEFT_MOUSE_DRAGGED_EVENT, (xCoord << 16) + yCoord);
            } else

            if (((e.getModifiersEx() & MouseEvent.BUTTON3_DOWN_MASK) > 0)) {
                BattleFieldView.this.updateAutoState(E18_RIGHT_MOUSE_DRAGGED_EVENT, (xCoord << 16) + yCoord);
            }
        }

        public void mouseReleased(MouseEvent e) {
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mouseMoved(MouseEvent e) {
            int xCoord = transScreenToViewX(e.getX());
            int yCoord = transScreenToViewY(e.getY());

            BattleFieldView.this.updateAutoState(E16_MOUSE_MOVED_EVENT, (xCoord << 16) + yCoord);
        }
    }

    public static void main(String[] args) {
        BattleFieldView view = BattleFieldView.createDefaultBattleFieldView();
        while (true) {
            view.battleField.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, Constants.TIME_DELTA, null);
            view.repaint();
        }
    }
}
