package wheelexp;

import physics.Constants;

import javax.swing.*;
import java.awt.*;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.geom.AffineTransform;
import java.text.NumberFormat;

/**
 * This class performs the visualization of the wheel of the mass
 * @author Boris Yartsev
 */
public class WheelPhysicsTest extends JFrame {

    private WheelPhysicsEmulator pe;

    public static final int DEFAULT_WHEEL_WIDTH = 200;
    public static final int DEFAULT_WHEEL_HEIGHT = 200;

    public static final int DEFAULT_CONTROL_WIDTH = 200;
    public static final int DEFAULT_CONTROL_HEIGHT = 150;

    private WheelDrawPanel wdpanel;
    private WheelControlPanel cpanel;

    public WheelPhysicsTest() {
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pe = new WheelPhysicsEmulator();
        wdpanel = new WheelDrawPanel();
        wdpanel.setPreferredSize(new Dimension(DEFAULT_WHEEL_WIDTH, DEFAULT_WHEEL_HEIGHT));
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(wdpanel, BorderLayout.CENTER);


        cpanel = new WheelControlPanel();
        cpanel.setPreferredSize(new Dimension(DEFAULT_CONTROL_WIDTH, DEFAULT_CONTROL_HEIGHT));
        getContentPane().add(cpanel, BorderLayout.SOUTH);

        pack();
        setResizable(false);
        setVisible(true);
    }

    /**
     * This class holds all the control of the physics
     */
    private class WheelControlPanel extends JPanel implements AdjustmentListener, ActionListener {

        //This scrollBar is from th  0 to 1.0
        private JScrollBar frictionScrollBar;
        private JLabel frictionLabel;

        //This scrollBar is from
        private JScrollBar momentScrollBar;
        private JLabel momentLabel;

        private JScrollBar spinFrictionCoeffScrollBar;
        private JLabel spinFrictionCoeffLabel;

        private NumberFormat format;

        private int min = 0;
        private int max = 100;

        private double maxFC = 1;
        private double maxM = 10;
        private double maxSFC = 0.1;

        public WheelControlPanel() {
            format = NumberFormat.getNumberInstance();
            format.setMaximumFractionDigits(2);

            setLayout(new GridBagLayout());
            GridBagConstraints gc = new GridBagConstraints();

            gc.insets = new Insets(1, 1, 1, 1);
            gc.gridx = 0;
            gc.gridy = 0;
            gc.gridwidth = 4;
            gc.gridheight = 1;
            gc.fill = GridBagConstraints.BOTH;
            gc.weightx = 1.0;
            gc.weighty = 1.0;

            frictionScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
            add(frictionScrollBar, gc);
            frictionScrollBar.setValues((int)((pe.getFrictionCoefficient() / maxFC) * max), 1, min, max + 1);
            frictionScrollBar.repaint();
            frictionScrollBar.addAdjustmentListener(this);

            gc.gridwidth = 1;
            gc.gridy = 1;
            frictionLabel = new JLabel();
            updateFrictionLabel(pe.getFrictionCoefficient());
            add(frictionLabel, gc);
            frictionLabel = new JLabel();
            updateFrictionLabel(pe.getFrictionCoefficient());
            add(frictionLabel, gc);

            gc.gridwidth = 4;
            gc.gridy = 2;
            momentScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
            add(momentScrollBar, gc);
            momentScrollBar.setValues((int) (((pe.getSpinMoment() + maxM) / maxM) * max / 2), 1, min, max + 1);
            momentScrollBar.repaint();
            momentScrollBar.addAdjustmentListener(this);

            gc.gridwidth = 1;
            gc.gridy = 3;
            momentLabel = new JLabel();
            updateSpinMomentLabel(pe.getSpinMoment());
            add(momentLabel, gc);

            gc.gridwidth = 4;
            gc.gridy = 4;
            spinFrictionCoeffScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
            add(spinFrictionCoeffScrollBar, gc);
            spinFrictionCoeffScrollBar.setValues((int) ((pe.getSpinForceCoefficient() / maxSFC) * max), 1, min, max + 1);
            spinFrictionCoeffScrollBar.repaint();
            spinFrictionCoeffScrollBar.addAdjustmentListener(this);

            gc.gridwidth = 1;
            gc.gridy = 5;
            spinFrictionCoeffLabel = new JLabel();
            updateSpinFrictionCoeffLabel(pe.getSpinForceCoefficient());
            add(spinFrictionCoeffLabel, gc);
        }

        private void updateFrictionLabel(double value) {
            String fc = format.format(value);
            frictionLabel.setText("Friction: " + fc);
        }

        private void updateSpinMomentLabel(double value) {
            String fc = format.format(value);
            momentLabel.setText("SpinMoment: " + fc);
        }

        private void updateSpinFrictionCoeffLabel(double value) {
            String sc = format.format(value);
            spinFrictionCoeffLabel.setText("SpinFrictionCoeff: " + sc);
        }

        public void adjustmentValueChanged(AdjustmentEvent e) {
            if (e.getSource() == frictionScrollBar) {
                double fc = ((double) frictionScrollBar.getValue() / max + Constants.EPS) * maxFC;
                updateFrictionLabel(fc);
                pe.setFrictionCoefficient(fc);
            } else

            if (e.getSource() == momentScrollBar) {
                double sm = (((double) momentScrollBar.getValue() - max / 2) / max + Constants.EPS) * 2 * maxM;
                updateSpinMomentLabel(sm);
                pe.setSpinMoment(sm);
            } else

            if (e.getSource() == spinFrictionCoeffScrollBar) {
                double sfc = ((double) spinFrictionCoeffScrollBar.getValue() / max + Constants.EPS) * maxSFC;
                updateSpinFrictionCoeffLabel(sfc);
                pe.setSpinForceCoefficient(sfc);
            }
        }

        public void actionPerformed(ActionEvent e) {

        }
    }

    private class WheelDrawPanel extends JPanel {
        private Image buffer;

        private Image wheelImage = Toolkit.getDefaultToolkit().getImage("wheel.png");
        private Image trackImage = Toolkit.getDefaultToolkit().getImage("track.png");
        private AffineTransform trans = new AffineTransform();


        /**
         * Todo refactor the code and remove all of these ugly constants
         * @param g
         */
        public void paintComponent(Graphics g) {
            int w = getSize().width;
            int h = getSize().height;

            if (buffer == null) {
                buffer = createImage(w, h);
            }

            Graphics2D ig = (Graphics2D) buffer.getGraphics();

            //fill everything with the white color
            ig.setColor(Color.WHITE);
            ig.fillRect(0, 0, w, h);

            trans.setToIdentity();
            trans.translate(50, 50);
            trans.rotate(-pe.getWheelAngle(),
                    wheelImage.getWidth(null) / 2, wheelImage.getHeight(null) / 2);

            ig.drawImage(wheelImage, trans, null);

            //the coordinate of the first track 100 is the
            int ftx = (int) Math.round(-100 * pe.getWheelPosition());

            ftx = (ftx > 0) ? (ftx % trackImage.getWidth(null)) :
                    (trackImage.getWidth(null) - (-ftx) % trackImage.getWidth(null));
            ftx = ftx - trackImage.getWidth(null);

            int n = w / trackImage.getWidth(null) + 3;

            for (int i = 0; i < n; i++) {
                trans.setToIdentity();
                trans.translate(ftx + i * trackImage.getWidth(null), 50 + wheelImage.getWidth(null));
                ig.drawImage(trackImage, trans, null);
            }


            double normFF = pe.getFrictionForceNormalized();
            int vector = (int) (wheelImage.getWidth(null) * normFF / 2);

            int xmax1 = 50 + wheelImage.getWidth(null) / 2;
            int xmax2 = 50 + wheelImage.getWidth(null) / 2;

            xmax2 = xmax2 + (int) (wheelImage.getWidth(null) * Math.signum(normFF) / 2);

            int x1 = 50 + wheelImage.getWidth(null) / 2;
            int x2 = 50 + wheelImage.getWidth(null) / 2 + vector;

            if (x2 < x1 || xmax2 < xmax1) {
                int t = x2; x2 = x1; x1 = t;
                t = xmax2; xmax2 = xmax1; xmax1 = t;
            }

            ig.setColor(Color.GREEN);
            ig.fillRect(xmax1, 50 + wheelImage.getHeight(null), xmax2 - xmax1, 3);
            ig.setColor(Color.RED);
            ig.fillRect(x1, 50 + wheelImage.getHeight(null), x2 - x1, 3);

            ig.setColor(Color.BLACK);
            if (pe.isSliding()) {
                ig.drawString("Slide!", 10, 180);
            }

            g.drawImage(buffer, 0, 0, null);
        }
    }

    private class WheelPhysicsEmulator {

        //Moment of inertia of the wheel
        private double IW = 1;

        //The radius of the wheel
        private double wheelRad = 0.5;

        //Moment track mass
        private double TM = 5;

        //the angle of the wheel
        private double wheelAngle = 0;

        //The speed of the wheel
        private double wheelSpeed = 0;

        //The position of the track
        private double wheelPosition = 0;

        //The speed of the track
        private double trackSpeed = 0;

        //The reaction force of the track to the wheel
        private double RF = 30;

        //The friction coefficient of the two surfaces
        private double FC = 0.0;

        //the force moment, that spins the wheel
        private double spinM = 0.0;

        //true if there is sliding between two surfaces
        private boolean sliding;


        private double SCF = 0.05;

        //The normalized value of the friction force, used to draw the vector on the picture
        private double FFNorm;

        public WheelPhysicsEmulator() {
            sliding = getSlidingDirection() != 0;
        }

        private int getSlidingDirection() {
            double wheelLowSpeed = wheelSpeed * wheelRad;

            if (wheelLowSpeed + trackSpeed > Constants.EPS) {
                return 1;
            } else

            if (wheelLowSpeed + trackSpeed < -Constants.EPS) {
                return -1;
            } else {
                return 0;
            }
        }

        public double getWheelAngle() {
            return wheelAngle;
        }

        public double getWheelPosition() {
            return wheelPosition;
        }

        public boolean isSliding() {
            return sliding;
        }

        /**
         * This method returns the frictionForce normalized by the maximum possible friction force
         * @return
         */
        public double getFrictionForceNormalized() {
            return FFNorm;
        }

        public double getFrictionCoefficient() {
            return FC;
        }

        public synchronized void setFrictionCoefficient(double FC) {
            this.FC = FC;
        }

        public double getSpinMoment() {
            return spinM;
        }

        public synchronized void setSpinMoment(double spinM) {
            this.spinM = spinM;
        }

        public double getSpinForceCoefficient() {
            return SCF;
        }

        public synchronized void setSpinForceCoefficient(double SCF) {
            this.SCF = SCF;
        }

        public synchronized void updateCoordinates2(double value) {

            //The positive value, denoting the maximum value of the friction force
            double maxFF = RF * FC;
            //The moment of the spinning friction force
            double spinFM = RF * SCF;

            if (Math.abs(wheelSpeed) > Constants.EPS) {
                spinFM = -Math.signum(wheelSpeed) * spinFM;
            } else {
                spinFM = 0;
            }

            int slDir = getSlidingDirection();

            //there is no sliding
            if (slDir == 0) {
                double realFF = -(spinM + spinFM) / (wheelRad + IW / (TM * wheelRad));

                //If the friction force moment can compensate the spinning moment and the cycling friction moment
                if (Math.abs(realFF) < maxFF + Constants.EPS) {
                    double realFM = realFF * wheelRad;
                    FFNorm = realFF / maxFF;

                    wheelSpeed += (spinM + spinFM + realFM) * value / IW;
                    trackSpeed = -wheelRad * wheelSpeed;
                } else {
                    realFF = -maxFF * Math.signum(spinM);
                    double realFM = -maxFF * wheelRad * Math.signum(spinM);

                    FFNorm = Math.signum(realFF);

                    wheelSpeed += (spinM + spinFM + realFM) * value / IW;
                    //one object influences the other object with the speed with the same module and oppisite direction
                    trackSpeed += realFF * value / TM;
                }
            }
            //The sliding is present
            else {
                //The direction of the friction
                double realFF = -maxFF * slDir;
                double realFM = realFF * wheelRad;

                FFNorm = -slDir;

                //The friction and the spinMoment and the spin friction moment affect the wheel
                wheelSpeed += (realFM + spinFM + spinM) * value / IW;
                trackSpeed += realFF * value / TM;

                int newSlDir = getSlidingDirection();

//                if the direction of the speed has changed, then it is equalized between wheel and track
                if (newSlDir != slDir) {
                    trackSpeed = -wheelSpeed * wheelRad;
                }
            }

            wheelAngle += value * wheelSpeed;
            wheelPosition += value * trackSpeed;
        }
    }

    public static void main(String[] args) {

        WheelPhysicsTest view = new WheelPhysicsTest();

        while (true) {
            view.pe.updateCoordinates2(Constants.TIME_DELTA);
            view.repaint();
        }
    }
}
